# encoding: utf8

from __future__ import print_function

import os

from nose.tools import eq_

import mapnik

from .utilities import (execution_path, get_unique_colors, pixel2channels,
                        run_all, side_by_side_image)


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def is_pre(color, alpha):
    return (color * 255.0 / alpha) <= 255


def debug_image(image, step=2):
    for x in range(0, image.width(), step):
        for y in range(0, image.height(), step):
            pixel = image.get_pixel(x, y)
            red, green, blue, alpha = pixel2channels(pixel)
            print(
                "rgba(%s,%s,%s,%s) at %s,%s" %
                (red, green, blue, alpha, x, y))


def replace_style(m, name, style):
    m.remove_style(name)
    m.append_style(name, style)

# note: it is impossible to know for all pixel colors
# we can only detect likely cases of non premultiplied colors


def validate_pixels_are_not_premultiplied(image):
    over_alpha = False
    transparent = True
    fully_opaque = True
    for x in range(0, image.width(), 2):
        for y in range(0, image.height(), 2):
            pixel = image.get_pixel(x, y)
            red, green, blue, alpha = pixel2channels(pixel)
            if alpha > 0:
                transparent = False
                if alpha < 255:
                    fully_opaque = False
                color_max = max(red, green, blue)
                if color_max > alpha:
                    over_alpha = True
    return over_alpha or transparent or fully_opaque


def validate_pixels_are_not_premultiplied2(image):
    looks_not_multiplied = False
    for x in range(0, image.width(), 2):
        for y in range(0, image.height(), 2):
            pixel = image.get_pixel(x, y)
            red, green, blue, alpha = pixel2channels(pixel)
            # each value of the color channels will never be bigger than that
            # of the alpha channel.
            if alpha > 0:
                if red > 0 and red > alpha:
                    print('red: %s, a: %s' % (red, alpha))
                    looks_not_multiplied = True
    return looks_not_multiplied


def validate_pixels_are_premultiplied(image):
    bad_pixels = []
    for x in range(0, image.width(), 2):
        for y in range(0, image.height(), 2):
            pixel = image.get_pixel(x, y)
            red, green, blue, alpha = pixel2channels(pixel)
            if alpha > 0:
                pixel = image.get_pixel(x, y)
                is_valid = ((0 <= red <= alpha) and is_pre(red, alpha)) \
                    and ((0 <= green <= alpha) and is_pre(green, alpha)) \
                    and ((0 <= blue <= alpha) and is_pre(blue, alpha)) \
                    and (alpha >= 0 and alpha <= 255)
                if not is_valid:
                    bad_pixels.append(
                        "rgba(%s,%s,%s,%s) at %s,%s" %
                        (red, green, blue, alpha, x, y))
    num_bad = len(bad_pixels)
    return (num_bad == 0, bad_pixels)


def test_compare_images():
    b = mapnik.Image.open('./images/support/b.png')
    b.premultiply()
    num_ops = len(mapnik.CompositeOp.names)
    successes = []
    fails = []
    for name in mapnik.CompositeOp.names:
        a = mapnik.Image.open('./images/support/a.png')
        a.premultiply()
        a.composite(b, getattr(mapnik.CompositeOp, name))
        actual = '/tmp/mapnik-comp-op-test-' + name + '.png'
        expected = 'images/composited/' + name + '.png'
        valid = validate_pixels_are_premultiplied(a)
        if not valid[0]:
            fails.append(
                '%s not validly premultiplied!:\n\t %s pixels (%s)' %
                (name, len(
                    valid[1]), valid[1][0]))
        a.demultiply()
        if not validate_pixels_are_not_premultiplied(a):
            fails.append('%s not validly demultiplied' % (name))
        a.save(actual, 'png32')
        if not os.path.exists(expected) or os.environ.get('UPDATE'):
            print('generating expected test image: %s' % expected)
            a.save(expected, 'png32')
        expected_im = mapnik.Image.open(expected)
        # compare them
        if a.tostring('png32') == expected_im.tostring('png32'):
            successes.append(name)
        else:
            fails.append(
                'failed comparing actual (%s) and expected(%s)' %
                (actual, 'tests/python_tests/' + expected))
            fail_im = side_by_side_image(expected_im, a)
            fail_im.save(
                '/tmp/mapnik-comp-op-test-' +
                name +
                '.fail.png',
                'png32')
    eq_(len(successes), num_ops, '\n' + '\n'.join(fails))
    b.demultiply()
    # b will be slightly modified by pre and then de multiplication rounding errors
    # TODO - write test to ensure the image is 99% the same.
    #expected_b = mapnik.Image.open('./images/support/b.png')
    # b.save('/tmp/mapnik-comp-op-test-original-mask.png')
    #eq_(b.tostring('png32'),expected_b.tostring('png32'), '/tmp/mapnik-comp-op-test-original-mask.png is no longer equivalent to original mask: ./images/support/b.png')


def test_pre_multiply_status():
    b = mapnik.Image.open('./images/support/b.png')
    # not premultiplied yet, should appear that way
    result = validate_pixels_are_not_premultiplied(b)
    eq_(result, True)
    # not yet premultiplied therefore should return false
    result = validate_pixels_are_premultiplied(b)
    eq_(result[0], False)
    # now actually premultiply the pixels
    b.premultiply()
    # now checking if premultiplied should succeed
    result = validate_pixels_are_premultiplied(b)
    eq_(result[0], True)
    # should now not appear to look not premultiplied
    result = validate_pixels_are_not_premultiplied(b)
    eq_(result, False)
    # now actually demultiply the pixels
    b.demultiply()
    # should now appear demultiplied
    result = validate_pixels_are_not_premultiplied(b)
    eq_(result, True)


def test_pre_multiply_status_of_map1():
    m = mapnik.Map(256, 256)
    im = mapnik.Image(m.width, m.height)
    eq_(validate_pixels_are_not_premultiplied(im), True)
    mapnik.render(m, im)
    eq_(validate_pixels_are_not_premultiplied(im), True)


def test_pre_multiply_status_of_map2():
    m = mapnik.Map(256, 256)
    m.background = mapnik.Color(1, 1, 1, 255)
    im = mapnik.Image(m.width, m.height)
    eq_(validate_pixels_are_not_premultiplied(im), True)
    mapnik.render(m, im)
    eq_(validate_pixels_are_not_premultiplied(im), True)

if 'shape' in mapnik.DatasourceCache.plugin_names():
    def test_style_level_comp_op():
        m = mapnik.Map(256, 256)
        mapnik.load_map(m, '../data/good_maps/style_level_comp_op.xml')
        m.zoom_all()
        successes = []
        fails = []
        for name in mapnik.CompositeOp.names:
            # find_style returns a copy of the style object
            style_markers = m.find_style("markers")
            style_markers.comp_op = getattr(mapnik.CompositeOp, name)
            # replace the original style with the modified one
            replace_style(m, "markers", style_markers)
            im = mapnik.Image(m.width, m.height)
            mapnik.render(m, im)
            actual = '/tmp/mapnik-style-comp-op-' + name + '.png'
            expected = 'images/style-comp-op/' + name + '.png'
            im.save(actual, 'png32')
            if not os.path.exists(expected) or os.environ.get('UPDATE'):
                print('generating expected test image: %s' % expected)
                im.save(expected, 'png32')
            expected_im = mapnik.Image.open(expected)
            # compare them
            if im.tostring('png32') == expected_im.tostring('png32'):
                successes.append(name)
            else:
                fails.append(
                    'failed comparing actual (%s) and expected(%s)' %
                    (actual, 'tests/python_tests/' + expected))
                fail_im = side_by_side_image(expected_im, im)
                fail_im.save(
                    '/tmp/mapnik-style-comp-op-' +
                    name +
                    '.fail.png',
                    'png32')
        eq_(len(fails), 0, '\n' + '\n'.join(fails))

    def test_style_level_opacity():
        m = mapnik.Map(512, 512)
        mapnik.load_map(
            m, '../data/good_maps/style_level_opacity_and_blur.xml')
        m.zoom_all()
        im = mapnik.Image(512, 512)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-style-level-opacity.png'
        expected = 'images/support/mapnik-style-level-opacity.png'
        im.save(actual, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'tests/python_tests/' + expected))


def test_rounding_and_color_expectations():
    m = mapnik.Map(1, 1)
    m.background = mapnik.Color('rgba(255,255,255,.4999999)')
    im = mapnik.Image(m.width, m.height)
    mapnik.render(m, im)
    eq_(get_unique_colors(im), ['rgba(255,255,255,127)'])
    m = mapnik.Map(1, 1)
    m.background = mapnik.Color('rgba(255,255,255,.5)')
    im = mapnik.Image(m.width, m.height)
    mapnik.render(m, im)
    eq_(get_unique_colors(im), ['rgba(255,255,255,128)'])
    im_file = mapnik.Image.open('../data/images/stripes_pattern.png')
    eq_(get_unique_colors(im_file), ['rgba(0,0,0,0)', 'rgba(74,74,74,255)'])
    # should have no effect
    im_file.premultiply()
    eq_(get_unique_colors(im_file), ['rgba(0,0,0,0)', 'rgba(74,74,74,255)'])
    im_file.apply_opacity(.5)
    # should have effect now that image has transparency
    im_file.premultiply()
    eq_(get_unique_colors(im_file), ['rgba(0,0,0,0)', 'rgba(37,37,37,127)'])
    # should restore to original nonpremultiplied colors
    im_file.demultiply()
    eq_(get_unique_colors(im_file), ['rgba(0,0,0,0)', 'rgba(74,74,74,127)'])


def test_background_image_and_background_color():
    m = mapnik.Map(8, 8)
    m.background = mapnik.Color('rgba(255,255,255,.5)')
    m.background_image = '../data/images/stripes_pattern.png'
    im = mapnik.Image(m.width, m.height)
    mapnik.render(m, im)
    eq_(get_unique_colors(im), ['rgba(255,255,255,128)', 'rgba(74,74,74,255)'])


def test_background_image_with_alpha_and_background_color():
    m = mapnik.Map(10, 10)
    m.background = mapnik.Color('rgba(255,255,255,.5)')
    m.background_image = '../data/images/yellow_half_trans.png'
    im = mapnik.Image(m.width, m.height)
    mapnik.render(m, im)
    eq_(get_unique_colors(im), ['rgba(255,255,85,191)'])


def test_background_image_with_alpha_and_background_color_against_composited_control():
    m = mapnik.Map(10, 10)
    m.background = mapnik.Color('rgba(255,255,255,.5)')
    m.background_image = '../data/images/yellow_half_trans.png'
    im = mapnik.Image(m.width, m.height)
    mapnik.render(m, im)
    # create and composite the expected result
    im1 = mapnik.Image(10, 10)
    im1.fill(mapnik.Color('rgba(255,255,255,.5)'))
    im1.premultiply()
    im2 = mapnik.Image(10, 10)
    im2.fill(mapnik.Color('rgba(255,255,0,.5)'))
    im2.premultiply()
    im1.composite(im2)
    im1.demultiply()
    # compare image rendered (compositing in `agg_renderer<T>::setup`)
    # vs image composited via python bindings
    #raise Todo("looks like we need to investigate PNG color rounding when saving")
    # eq_(get_unique_colors(im),get_unique_colors(im1))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import os
from glob import glob

import mapnik

from .utilities import execution_path, run_all


default_logging_severity = mapnik.logger.get_severity()


def setup():
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def teardown():
    mapnik.logger.set_severity(default_logging_severity)

plugin_mapping = {
    '.csv': ['csv'],
    '.json': ['geojson', 'ogr'],
    '.tif': ['gdal'],
    #'.tif' : ['gdal','raster'],
    '.kml': ['ogr'],
    '.gpx': ['ogr'],
    '.vrt': ['gdal']
}


def test_opening_data():
    # https://github.com/mapbox/mapnik-test-data
    # cd tests/data
    # git clone --depth 1 https://github.com/mapbox/mapnik-test-data
    if os.path.exists('../data/mapnik-test-data/'):
        files = glob('../data/mapnik-test-data/data/*/*.*')
        for filepath in files:
            ext = os.path.splitext(filepath)[1]
            if plugin_mapping.get(ext):
                # print 'testing opening %s' % filepath
                if 'topo' in filepath:
                    kwargs = {'type': 'ogr', 'file': filepath}
                    kwargs['layer_by_index'] = 0
                    try:
                        mapnik.Datasource(**kwargs)
                    except Exception as e:
                        print('could not open, %s: %s' % (kwargs, e))
                else:
                    for plugin in plugin_mapping[ext]:
                        kwargs = {'type': plugin, 'file': filepath}
                        if plugin is 'ogr':
                            kwargs['layer_by_index'] = 0
                        try:
                            mapnik.Datasource(**kwargs)
                        except Exception as e:
                            print('could not open, %s: %s' % (kwargs, e))
            # else:
            #    print 'skipping opening %s' % filepath

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, get_unique_colors, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_dataraster_coloring():
    srs = '+init=epsg:32630'
    lyr = mapnik.Layer('dataraster')
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        lyr.datasource = mapnik.Gdal(
            file='../data/raster/dataraster.tif',
            band=1,
        )
        lyr.srs = srs
        _map = mapnik.Map(256, 256, srs)
        style = mapnik.Style()
        rule = mapnik.Rule()
        sym = mapnik.RasterSymbolizer()
        # Assigning a colorizer to the RasterSymbolizer tells the later
        # that it should use it to colorize the raw data raster
        colorizer = mapnik.RasterColorizer(
            mapnik.COLORIZER_DISCRETE,
            mapnik.Color("transparent"))

        for value, color in [
            (0, "#0044cc"),
            (10, "#00cc00"),
            (20, "#ffff00"),
            (30, "#ff7f00"),
            (40, "#ff0000"),
            (50, "#ff007f"),
            (60, "#ff00ff"),
            (70, "#cc00cc"),
            (80, "#990099"),
            (90, "#660066"),
            (200, "transparent"),
        ]:
            colorizer.add_stop(value, mapnik.Color(color))
        sym.colorizer = colorizer
        rule.symbols.append(sym)
        style.rules.append(rule)
        _map.append_style('foo', style)
        lyr.styles.append('foo')
        _map.layers.append(lyr)
        _map.zoom_to_box(lyr.envelope())

        im = mapnik.Image(_map.width, _map.height)
        mapnik.render(_map, im)
        expected_file = './images/support/dataraster_coloring.png'
        actual_file = '/tmp/' + os.path.basename(expected_file)
        im.save(actual_file, 'png32')
        if not os.path.exists(expected_file) or os.environ.get('UPDATE'):
            im.save(expected_file, 'png32')
        actual = mapnik.Image.open(actual_file)
        expected = mapnik.Image.open(expected_file)
        eq_(actual.tostring('png32'),
            expected.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual_file,
                                                                expected_file))


def test_dataraster_query_point():
    srs = '+init=epsg:32630'
    lyr = mapnik.Layer('dataraster')
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        lyr.datasource = mapnik.Gdal(
            file='../data/raster/dataraster.tif',
            band=1,
        )
        lyr.srs = srs
        _map = mapnik.Map(256, 256, srs)
        _map.layers.append(lyr)

        x, y = 556113.0, 4381428.0  # center of extent of raster
        _map.zoom_all()
        features = list(_map.query_point(0, x, y))
        assert len(features) == 1
        feat = features[0]
        center = feat.envelope().center()
        assert center.x == x and center.y == y, center
        value = feat['value']
        assert value == 18.0, value

        # point inside map extent but outside raster extent
        current_box = _map.envelope()
        current_box.expand_to_include(-427417, 4477517)
        _map.zoom_to_box(current_box)
        features = _map.query_point(0, -427417, 4477517)
        assert len(list(features)) == 0

        # point inside raster extent with nodata
        features = _map.query_point(0, 126850, 4596050)
        assert len(list(features)) == 0


def test_load_save_map():
    map = mapnik.Map(256, 256)
    in_map = "../data/good_maps/raster_symbolizer.xml"
    try:
        mapnik.load_map(map, in_map)

        out_map = mapnik.save_map_to_string(map)
        assert 'RasterSymbolizer' in out_map
        assert 'RasterColorizer' in out_map
        assert 'stop' in out_map
    except RuntimeError as e:
        # only test datasources that we have installed
        if not 'Could not create datasource' in str(e):
            raise RuntimeError(str(e))


def test_raster_with_alpha_blends_correctly_with_background():
    WIDTH = 500
    HEIGHT = 500

    map = mapnik.Map(WIDTH, HEIGHT)
    WHITE = mapnik.Color(255, 255, 255)
    map.background = WHITE

    style = mapnik.Style()
    rule = mapnik.Rule()
    symbolizer = mapnik.RasterSymbolizer()
    symbolizer.scaling = mapnik.scaling_method.BILINEAR

    rule.symbols.append(symbolizer)
    style.rules.append(rule)

    map.append_style('raster_style', style)

    map_layer = mapnik.Layer('test_layer')
    filepath = '../data/raster/white-alpha.png'
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        map_layer.datasource = mapnik.Gdal(file=filepath)
        map_layer.styles.append('raster_style')
        map.layers.append(map_layer)

        map.zoom_all()

        mim = mapnik.Image(WIDTH, HEIGHT)

        mapnik.render(map, mim)
        mim.tostring()
        # All white is expected
        eq_(get_unique_colors(mim), ['rgba(254,254,254,255)'])


def test_raster_warping():
    lyrSrs = "+init=epsg:32630"
    mapSrs = '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
    lyr = mapnik.Layer('dataraster', lyrSrs)
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        lyr.datasource = mapnik.Gdal(
            file='../data/raster/dataraster.tif',
            band=1,
        )
        sym = mapnik.RasterSymbolizer()
        sym.colorizer = mapnik.RasterColorizer(
            mapnik.COLORIZER_DISCRETE, mapnik.Color(255, 255, 0))
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style = mapnik.Style()
        style.rules.append(rule)
        _map = mapnik.Map(256, 256, mapSrs)
        _map.append_style('foo', style)
        lyr.styles.append('foo')
        _map.layers.append(lyr)
        map_proj = mapnik.Projection(mapSrs)
        layer_proj = mapnik.Projection(lyrSrs)
        prj_trans = mapnik.ProjTransform(map_proj,
                                         layer_proj)
        _map.zoom_to_box(prj_trans.backward(lyr.envelope()))

        im = mapnik.Image(_map.width, _map.height)
        mapnik.render(_map, im)
        expected_file = './images/support/raster_warping.png'
        actual_file = '/tmp/' + os.path.basename(expected_file)
        im.save(actual_file, 'png32')
        if not os.path.exists(expected_file) or os.environ.get('UPDATE'):
            im.save(expected_file, 'png32')
        actual = mapnik.Image.open(actual_file)
        expected = mapnik.Image.open(expected_file)
        eq_(actual.tostring('png32'),
            expected.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual_file,
                                                                expected_file))


def test_raster_warping_does_not_overclip_source():
    lyrSrs = "+init=epsg:32630"
    mapSrs = '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
    lyr = mapnik.Layer('dataraster', lyrSrs)
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        lyr.datasource = mapnik.Gdal(
            file='../data/raster/dataraster.tif',
            band=1,
        )
        sym = mapnik.RasterSymbolizer()
        sym.colorizer = mapnik.RasterColorizer(
            mapnik.COLORIZER_DISCRETE, mapnik.Color(255, 255, 0))
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style = mapnik.Style()
        style.rules.append(rule)
        _map = mapnik.Map(256, 256, mapSrs)
        _map.background = mapnik.Color('white')
        _map.append_style('foo', style)
        lyr.styles.append('foo')
        _map.layers.append(lyr)
        _map.zoom_to_box(mapnik.Box2d(3, 42, 4, 43))

        im = mapnik.Image(_map.width, _map.height)
        mapnik.render(_map, im)
        expected_file = './images/support/raster_warping_does_not_overclip_source.png'
        actual_file = '/tmp/' + os.path.basename(expected_file)
        im.save(actual_file, 'png32')
        if not os.path.exists(expected_file) or os.environ.get('UPDATE'):
            im.save(expected_file, 'png32')
        actual = mapnik.Image.open(actual_file)
        expected = mapnik.Image.open(expected_file)
        eq_(actual.tostring('png32'),
            expected.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual_file,
                                                                expected_file))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function

import os

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if mapnik.has_webp():
    tmp_dir = '/tmp/mapnik-webp/'
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)

    opts = [
        'webp',
        'webp:method=0',
        'webp:method=6',
        'webp:quality=64',
        'webp:alpha=false',
        'webp:partitions=3',
        'webp:preprocessing=1',
        'webp:partition_limit=50',
        'webp:pass=10',
        'webp:alpha_quality=50',
        'webp:alpha_filtering=2',
        'webp:alpha_compression=0',
        'webp:autofilter=0',
        'webp:filter_type=1:autofilter=1',
        'webp:filter_sharpness=4',
        'webp:filter_strength=50',
        'webp:sns_strength=50',
        'webp:segments=3',
        'webp:target_PSNR=.5',
        'webp:target_size=100'
    ]

    def gen_filepath(name, format):
        return os.path.join('images/support/encoding-opts',
                            name + '-' + format.replace(":", "+") + '.webp')

    def test_quality_threshold():
        im = mapnik.Image(256, 256)
        im.tostring('webp:quality=99.99000')
        im.tostring('webp:quality=0')
        im.tostring('webp:quality=0.001')

    @raises(RuntimeError)
    def test_quality_threshold_invalid():
        im = mapnik.Image(256, 256)
        im.tostring('webp:quality=101')

    @raises(RuntimeError)
    def test_quality_threshold_invalid2():
        im = mapnik.Image(256, 256)
        im.tostring('webp:quality=-1')

    @raises(RuntimeError)
    def test_quality_threshold_invalid3():
        im = mapnik.Image(256, 256)
        im.tostring('webp:quality=101.1')

    generate = os.environ.get('UPDATE')

    def test_expected_encodings():
        fails = []
        try:
            for opt in opts:
                im = mapnik.Image(256, 256)
                expected = gen_filepath('blank', opt)
                actual = os.path.join(tmp_dir, os.path.basename(expected))
                if generate or not os.path.exists(expected):
                    print('generating expected image', expected)
                    im.save(expected, opt)
                im.save(actual, opt)
                try:
                    expected_bytes = mapnik.Image.open(expected).tostring()
                except RuntimeError:
                    # this will happen if libweb is old, since it cannot open
                    # images created by more recent webp
                    print(
                        'warning, cannot open webp expected image (your libwebp is likely too old)')
                    continue
                if mapnik.Image.open(actual).tostring() != expected_bytes:
                    fails.append(
                        '%s (actual) not == to %s (expected)' %
                        (actual, expected))

            for opt in opts:
                im = mapnik.Image(256, 256)
                im.fill(mapnik.Color('green'))
                expected = gen_filepath('solid', opt)
                actual = os.path.join(tmp_dir, os.path.basename(expected))
                if generate or not os.path.exists(expected):
                    print('generating expected image', expected)
                    im.save(expected, opt)
                im.save(actual, opt)
                try:
                    expected_bytes = mapnik.Image.open(expected).tostring()
                except RuntimeError:
                    # this will happen if libweb is old, since it cannot open
                    # images created by more recent webp
                    print(
                        'warning, cannot open webp expected image (your libwebp is likely too old)')
                    continue
                if mapnik.Image.open(actual).tostring() != expected_bytes:
                    fails.append(
                        '%s (actual) not == to %s (expected)' %
                        (actual, expected))

            for opt in opts:
                im = mapnik.Image.open(
                    'images/support/transparency/aerial_rgba.png')
                expected = gen_filepath('aerial_rgba', opt)
                actual = os.path.join(tmp_dir, os.path.basename(expected))
                if generate or not os.path.exists(expected):
                    print('generating expected image', expected)
                    im.save(expected, opt)
                im.save(actual, opt)
                try:
                    expected_bytes = mapnik.Image.open(expected).tostring()
                except RuntimeError:
                    # this will happen if libweb is old, since it cannot open
                    # images created by more recent webp
                    print(
                        'warning, cannot open webp expected image (your libwebp is likely too old)')
                    continue
                if mapnik.Image.open(actual).tostring() != expected_bytes:
                    fails.append(
                        '%s (actual) not == to %s (expected)' %
                        (actual, expected))
            # disabled to avoid failures on ubuntu when using old webp packages
            # eq_(fails,[],'\n'+'\n'.join(fails))
        except RuntimeError as e:
            print(e)

    def test_transparency_levels():
        try:
            # create partial transparency image
            im = mapnik.Image(256, 256)
            im.fill(mapnik.Color('rgba(255,255,255,.5)'))
            c2 = mapnik.Color('rgba(255,255,0,.2)')
            c3 = mapnik.Color('rgb(0,255,255)')
            for y in range(0, int(im.height() / 2)):
                for x in range(0, int(im.width() / 2)):
                    im.set_pixel(x, y, c2)
            for y in range(int(im.height() / 2), im.height()):
                for x in range(int(im.width() / 2), im.width()):
                    im.set_pixel(x, y, c3)

            t0 = tmp_dir + 'white0-actual.webp'

            # octree
            format = 'webp'
            expected = 'images/support/transparency/white0.webp'
            if generate or not os.path.exists(expected):
                im.save('images/support/transparency/white0.webp')
            im.save(t0, format)
            im_in = mapnik.Image.open(t0)
            t0_len = len(im_in.tostring(format))
            try:
                expected_bytes = mapnik.Image.open(expected).tostring(format)
            except RuntimeError:
                # this will happen if libweb is old, since it cannot open
                # images created by more recent webp
                print(
                    'warning, cannot open webp expected image (your libwebp is likely too old)')
                return
            eq_(t0_len, len(expected_bytes))
        except RuntimeError as e:
            print(e)


if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import glob
import os
import tempfile

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


default_logging_severity = mapnik.logger.get_severity()


def setup():
    # make the tests silent to suppress unsupported params from harfbuzz tests
    # TODO: remove this after harfbuzz branch merges
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def teardown():
    mapnik.logger.set_severity(default_logging_severity)


def compare_map(xml):
    m = mapnik.Map(256, 256)
    absolute_base = os.path.abspath(os.path.dirname(xml))
    try:
        mapnik.load_map(m, xml, False, absolute_base)
    except RuntimeError as e:
        # only test datasources that we have installed
        if not 'Could not create datasource' in str(e) \
           and not 'could not connect' in str(e):
            raise RuntimeError(str(e))
        return
    (handle, test_map) = tempfile.mkstemp(
        suffix='.xml', prefix='mapnik-temp-map1-')
    os.close(handle)
    (handle, test_map2) = tempfile.mkstemp(
        suffix='.xml', prefix='mapnik-temp-map2-')
    os.close(handle)
    if os.path.exists(test_map):
        os.remove(test_map)
    mapnik.save_map(m, test_map)
    new_map = mapnik.Map(256, 256)
    mapnik.load_map(new_map, test_map, False, absolute_base)
    with open(test_map2, 'w') as f:
        f.write(mapnik.save_map_to_string(new_map))
    diff = ' diff -u %s %s' % (os.path.abspath(test_map),
                               os.path.abspath(test_map2))
    try:
        with open(test_map) as f1:
            with open(test_map2) as f2:
                eq_(f1.read(), f2.read())
    except AssertionError as e:
        raise AssertionError(
            'serialized map "%s" not the same after being reloaded, \ncompare with command:\n\n$%s' %
            (xml, diff))

    if os.path.exists(test_map):
        os.remove(test_map)
    else:
        # Fail, the map wasn't written
        return False


def test_compare_map():
    good_maps = glob.glob("../data/good_maps/*.xml")
    good_maps = [os.path.normpath(p) for p in good_maps]
    # remove one map that round trips CDATA differently, but this is okay
    ignorable = os.path.join('..', 'data', 'good_maps', 'empty_parameter2.xml')
    good_maps.remove(ignorable)
    for m in good_maps:
        compare_map(m)

    for m in glob.glob("../visual_tests/styles/*.xml"):
        compare_map(m)

# TODO - enforce that original xml does not equal first saved xml


def test_compare_map_deprecations():
    dep = glob.glob("../data/deprecated_maps/*.xml")
    dep = [os.path.normpath(p) for p in dep]
    for m in dep:
        compare_map(m)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from nose.tools import eq_

import mapnik

from .utilities import run_all

# geojson box of the world
geojson = {"type": "Feature",
           "properties": {},
           "geometry": {"type": "Polygon",
                        "coordinates": [[[-17963313.143242701888084,
                                          -6300857.11560364998877],
                                         [-17963313.143242701888084,
                                          13071343.332991421222687],
                                         [7396658.353099936619401,
                                          13071343.332991421222687],
                                         [7396658.353099936619401,
                                          -6300857.11560364998877],
                                         [-17963313.143242701888084,
                                          -6300857.11560364998877]]]}}


def test_that_coordinates_do_not_overflow_and_polygon_is_rendered_memory():
    expected_color = mapnik.Color('white')
    projection = '+init=epsg:4326'
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    feat = mapnik.Feature.from_geojson(json.dumps(geojson), context)
    ds.add_feature(feat)
    s = mapnik.Style()
    r = mapnik.Rule()
    sym = mapnik.PolygonSymbolizer()
    sym.fill = expected_color
    r.symbols.append(sym)
    s.rules.append(r)
    lyr = mapnik.Layer('Layer', projection)
    lyr.datasource = ds
    lyr.styles.append('style')
    m = mapnik.Map(256, 256, projection)
    m.background_color = mapnik.Color('green')
    m.append_style('style', s)
    m.layers.append(lyr)
    # 17/20864/45265.png
    m.zoom_to_box(mapnik.Box2d(-13658379.710221574,
                               6197514.253362091, -13657768.213995293, 6198125.749588372))
    # works 15/5216/11316.png
    # m.zoom_to_box(mapnik.Box2d(-13658379.710221574,6195679.764683247,-13655933.72531645,6198125.749588372))
    im = mapnik.Image(256, 256)
    mapnik.render(m, im)
    eq_(im.get_pixel(128, 128), expected_color.packed())


def test_that_coordinates_do_not_overflow_and_polygon_is_rendered_csv():
    expected_color = mapnik.Color('white')
    projection = '+init=epsg:4326'
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    feat = mapnik.Feature.from_geojson(json.dumps(geojson), context)
    ds.add_feature(feat)
    geojson_string = "geojson\n'%s'" % json.dumps(geojson['geometry'])
    ds = mapnik.Datasource(**{'type': 'csv', 'inline': geojson_string})
    s = mapnik.Style()
    r = mapnik.Rule()
    sym = mapnik.PolygonSymbolizer()
    sym.fill = expected_color
    r.symbols.append(sym)
    s.rules.append(r)
    lyr = mapnik.Layer('Layer', projection)
    lyr.datasource = ds
    lyr.styles.append('style')
    m = mapnik.Map(256, 256, projection)
    m.background_color = mapnik.Color('green')
    m.append_style('style', s)
    m.layers.append(lyr)
    # 17/20864/45265.png
    m.zoom_to_box(mapnik.Box2d(-13658379.710221574,
                               6197514.253362091, -13657768.213995293, 6198125.749588372))
    # works 15/5216/11316.png
    # m.zoom_to_box(mapnik.Box2d(-13658379.710221574,6195679.764683247,-13655933.72531645,6198125.749588372))
    im = mapnik.Image(256, 256)
    mapnik.render(m, im)
    eq_(im.get_pixel(128, 128), expected_color.packed())

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



"""Scale helpers functions."""

import math


def any_scale(scale):
    """Scale helper function that allows any scale."""
    return scale

def sequence_scale(scale, scale_sequence):
    """Sequence scale helper, this rounds scale to a 'sensible' value."""
    factor = math.floor(math.log10(scale))
    norm = scale / (10**factor)

    for s in scale_sequence:
        if norm <= s:
            return s * 10**factor

    return scale_sequence[0] * 10**(factor + 1)

def default_scale(scale):
    """Default scale helper, this rounds scale to a 'sensible' value."""
    return sequence_scale(scale, (1, 1.25, 1.5, 1.75, 2, 2.5, 3, 4, 5, 6, 7.5, 8, 9, 10))

def deg_min_sec_scale(scale):
    for x in (1.0 / 3600,
              2.0 / 3600,
              5.0 / 3600,
              10.0 / 3600,
              30.0 / 3600,
              1.0 / 60,
              2.0 / 60,
              5.0 / 60,
              10.0 / 60,
              30.0 / 60,
              1,
              2,
              5,
              10,
              30,
              60
              ):
        if scale < x:
            return x
    else:
        return x



#
# This file is part of Mapnik (c++ mapping toolkit)
# Copyright (C) 2015 Artem Pavlenko
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

"""Mapnik Python module.

Boost Python bindings to the Mapnik C++ shared library.

Several things happen when you do:

    >>> import mapnik

 1) Mapnik C++ objects are imported via the '__init__.py' from the '_mapnik.so' shared object
    (_mapnik.pyd on win) which references libmapnik.so (linux), libmapnik.dylib (mac), or
    mapnik.dll (win32).

 2) The paths to the input plugins and font directories are imported from the 'paths.py'
    file which was constructed and installed during SCons installation.

 3) All available input plugins and TrueType fonts are automatically registered.

 4) Boost Python metaclass injectors are used in the '__init__.py' to extend several
    objects adding extra convenience when accessed via Python.

"""

import itertools
import os
import warnings
try:
    import json
except ImportError:
    import simplejson as json


def bootstrap_env():
    """
    If an optional settings file exists, inherit its
    environment settings before loading the mapnik library.

    This feature is intended for customized packages of mapnik.

    The settings file should be a python file with an 'env' variable
    that declares a dictionary of key:value pairs to push into the
    global process environment, if not already set, like:

        env = {'ICU_DATA':'/usr/local/share/icu/'}
    """
    if os.path.exists(os.path.join(
            os.path.dirname(__file__), 'mapnik_settings.py')):
        from .mapnik_settings import env
        process_keys = os.environ.keys()
        for key, value in env.items():
            if key not in process_keys:
                os.environ[key] = value

bootstrap_env()

from ._mapnik import *

# The base Boost.Python class
BoostPythonMetaclass = Coord.__class__


class _MapnikMetaclass(BoostPythonMetaclass):

    def __init__(self, name, bases, dict):
        for b in bases:
            if type(b) not in (self, type):
                for k, v in list(dict.items()):
                    if hasattr(b, k):
                        setattr(b, '_c_' + k, getattr(b, k))
                    setattr(b, k, v)
        return type.__init__(self, name, bases, dict)

# metaclass injector compatible with both python 2 and 3
# http://mikewatkins.ca/2008/11/29/python-2-and-3-metaclasses/
def _injector() :
    return _MapnikMetaclass('_injector', (object, ), {})


def Filter(*args, **kwargs):
    warnings.warn("'Filter' is deprecated and will be removed in Mapnik 3.x, use 'Expression' instead",
                  DeprecationWarning, 2)
    return Expression(*args, **kwargs)


class Envelope(Box2d):

    def __init__(self, *args, **kwargs):
        warnings.warn("'Envelope' is deprecated and will be removed in Mapnik 3.x, use 'Box2d' instead",
                      DeprecationWarning, 2)
        Box2d.__init__(self, *args, **kwargs)


class _Coord(Coord, _injector()):
    """
    Represents a point with two coordinates (either lon/lat or x/y).

    Following operators are defined for Coord:

    Addition and subtraction of Coord objects:

    >>> Coord(10, 10) + Coord(20, 20)
    Coord(30.0, 30.0)
    >>> Coord(10, 10) - Coord(20, 20)
    Coord(-10.0, -10.0)

    Addition, subtraction, multiplication and division between
    a Coord and a float:

    >>> Coord(10, 10) + 1
    Coord(11.0, 11.0)
    >>> Coord(10, 10) - 1
    Coord(-9.0, -9.0)
    >>> Coord(10, 10) * 2
    Coord(20.0, 20.0)
    >>> Coord(10, 10) / 2
    Coord(5.0, 5.0)

    Equality of coords (as pairwise equality of components):
    >>> Coord(10, 10) is Coord(10, 10)
    False
    >>> Coord(10, 10) == Coord(10, 10)
    True
    """

    def __repr__(self):
        return 'Coord(%s,%s)' % (self.x, self.y)

    def forward(self, projection):
        """
        Projects the point from the geographic coordinate
        space  into the cartesian space. The x component is
        considered to be longitude, the y component the
        latitude.

        Returns the easting (x) and northing (y) as a
        coordinate pair.

        Example: Project the geographic coordinates of the
                 city center of Stuttgart into the local
                 map projection (GK Zone 3/DHDN, EPSG 31467)
        >>> p = Projection('+init=epsg:31467')
        >>> Coord(9.1, 48.7).forward(p)
        Coord(3507360.12813,5395719.2749)
        """
        return forward_(self, projection)

    def inverse(self, projection):
        """
        Projects the point from the cartesian space
        into the geographic space. The x component is
        considered to be the easting, the y component
        to be the northing.

        Returns the longitude (x) and latitude (y) as a
        coordinate pair.

        Example: Project the cartesian coordinates of the
                 city center of Stuttgart in the local
                 map projection (GK Zone 3/DHDN, EPSG 31467)
                 into geographic coordinates:
        >>> p = Projection('+init=epsg:31467')
        >>> Coord(3507360.12813,5395719.2749).inverse(p)
        Coord(9.1, 48.7)
        """
        return inverse_(self, projection)


class _Box2d(Box2d, _injector()):
    """
    Represents a spatial envelope (i.e. bounding box).


    Following operators are defined for Box2d:

    Addition:
    e1 + e2 is equivalent to e1.expand_to_include(e2) but yields
    a new envelope instead of modifying e1

    Subtraction:
    Currently e1 - e2 returns e1.

    Multiplication and division with floats:
    Multiplication and division change the width and height of the envelope
    by the given factor without modifying its center..

    That is, e1 * x is equivalent to:
           e1.width(x * e1.width())
           e1.height(x * e1.height()),
    except that a new envelope is created instead of modifying e1.

    e1 / x is equivalent to e1 * (1.0/x).

    Equality: two envelopes are equal if their corner points are equal.
    """

    def __repr__(self):
        return 'Box2d(%s,%s,%s,%s)' % \
            (self.minx, self.miny, self.maxx, self.maxy)

    def forward(self, projection):
        """
        Projects the envelope from the geographic space
        into the cartesian space by projecting its corner
        points.

        See also:
           Coord.forward(self, projection)
        """
        return forward_(self, projection)

    def inverse(self, projection):
        """
        Projects the envelope from the cartesian space
        into the geographic space by projecting its corner
        points.

        See also:
          Coord.inverse(self, projection).
        """
        return inverse_(self, projection)


class _Projection(Projection, _injector()):

    def __repr__(self):
        return "Projection('%s')" % self.params()

    def forward(self, obj):
        """
        Projects the given object (Box2d or Coord)
        from the geographic space into the cartesian space.

        See also:
          Box2d.forward(self, projection),
          Coord.forward(self, projection).
        """
        return forward_(obj, self)

    def inverse(self, obj):
        """
        Projects the given object (Box2d or Coord)
        from the cartesian space into the geographic space.

        See also:
          Box2d.inverse(self, projection),
          Coord.inverse(self, projection).
        """
        return inverse_(obj, self)


class _Feature(Feature, _injector()):
    __geo_interface__ = property(lambda self: json.loads(self.to_geojson()))


class _Geometry(Geometry, _injector()):
    __geo_interface__ = property(lambda self: json.loads(self.to_geojson()))


class _Datasource(Datasource, _injector()):

    def featureset(self, fields = None, variables = {}):
        query = Query(self.envelope())
        query.set_variables(variables)
        attributes = fields or self.fields()
        for fld in attributes:
            query.add_property_name(fld)
        return self.features(query)

    def __iter__(self, fields = None, variables = {}):
        return self.featureset(fields, variables)
    # backward caps helper
    def all_features(self, fields=None, variables={}):
        return self.__iter__(fields, variables)


class _Color(Color, _injector()):

    def __repr__(self):
        return "Color(R=%d,G=%d,B=%d,A=%d)" % (self.r, self.g, self.b, self.a)


class _SymbolizerBase(SymbolizerBase, _injector()):
    # back compatibility

    @property
    def filename(self):
        return self['file']

    @filename.setter
    def filename(self, val):
        self['file'] = val


def _add_symbol_method_to_symbolizers(vars=globals()):

    def symbol_for_subcls(self):
        return self

    def symbol_for_cls(self):
        return getattr(self, self.type())()

    for name, obj in vars.items():
        if name.endswith('Symbolizer') and not name.startswith('_'):
            if name == 'Symbolizer':
                symbol = symbol_for_cls
            else:
                symbol = symbol_for_subcls
            type('dummy', (obj, _injector()), {'symbol': symbol})
_add_symbol_method_to_symbolizers()


def Datasource(**keywords):
    """Wrapper around CreateDatasource.

    Create a Mapnik Datasource using a dictionary of parameters.

    Keywords must include:

      type='plugin_name' # e.g. type='gdal'

    See the convenience factory methods of each input plugin for
    details on additional required keyword arguments.

    """

    return CreateDatasource(keywords)

# convenience factory methods


def Shapefile(**keywords):
    """Create a Shapefile Datasource.

    Required keyword arguments:
      file -- path to shapefile without extension

    Optional keyword arguments:
      base -- path prefix (default None)
      encoding -- file encoding (default 'utf-8')

    >>> from mapnik import Shapefile, Layer
    >>> shp = Shapefile(base='/home/mapnik/data',file='world_borders')
    >>> lyr = Layer('Shapefile Layer')
    >>> lyr.datasource = shp

    """
    keywords['type'] = 'shape'
    return CreateDatasource(keywords)


def CSV(**keywords):
    """Create a CSV Datasource.

    Required keyword arguments:
      file -- path to csv

    Optional keyword arguments:
      inline -- inline CSV string (if provided 'file' argument will be ignored and non-needed)
      base -- path prefix (default None)
      encoding -- file encoding (default 'utf-8')
      row_limit -- integer limit of rows to return (default: 0)
      strict -- throw an error if an invalid row is encountered
      escape -- The escape character to use for parsing data
      quote -- The quote character to use for parsing data
      separator -- The separator character to use for parsing data
      headers -- A comma separated list of header names that can be set to add headers to data that lacks them
      filesize_max -- The maximum filesize in MB that will be accepted

    >>> from mapnik import CSV
    >>> csv = CSV(file='test.csv')

    >>> from mapnik import CSV
    >>> csv = CSV(inline='''wkt,Name\n"POINT (120.15 48.47)","Winthrop, WA"''')

    For more information see https://github.com/mapnik/mapnik/wiki/CSV-Plugin

    """
    keywords['type'] = 'csv'
    return CreateDatasource(keywords)


def GeoJSON(**keywords):
    """Create a GeoJSON Datasource.

    Required keyword arguments:
      file -- path to json

    Optional keyword arguments:
      encoding -- file encoding (default 'utf-8')
      base -- path prefix (default None)

    >>> from mapnik import GeoJSON
    >>> geojson = GeoJSON(file='test.json')

    """
    keywords['type'] = 'geojson'
    return CreateDatasource(keywords)


def PostGIS(**keywords):
    """Create a PostGIS Datasource.

    Required keyword arguments:
      dbname -- database name to connect to
      table -- table name or subselect query

      *Note: if using subselects for the 'table' value consider also
       passing the 'geometry_field' and 'srid' and 'extent_from_subquery'
       options and/or specifying the 'geometry_table' option.

    Optional db connection keyword arguments:
      user -- database user to connect as (default: see postgres docs)
      password -- password for database user (default: see postgres docs)
      host -- postgres hostname (default: see postgres docs)
      port -- postgres port (default: see postgres docs)
      initial_size -- integer size of connection pool (default: 1)
      max_size -- integer max of connection pool (default: 10)
      persist_connection -- keep connection open (default: True)

    Optional table-level keyword arguments:
      extent -- manually specified data extent (comma delimited string, default: None)
      estimate_extent -- boolean, direct PostGIS to use the faster, less accurate `estimate_extent` over `extent` (default: False)
      extent_from_subquery -- boolean, direct Mapnik to query Postgis for the extent of the raw 'table' value (default: uses 'geometry_table')
      geometry_table -- specify geometry table to use to look up metadata (default: automatically parsed from 'table' value)
      geometry_field -- specify geometry field to use (default: first entry in geometry_columns)
      srid -- specify srid to use (default: auto-detected from geometry_field)
      row_limit -- integer limit of rows to return (default: 0)
      cursor_size -- integer size of binary cursor to use (default: 0, no binary cursor is used)

    >>> from mapnik import PostGIS, Layer
    >>> params = dict(dbname=env['MAPNIK_NAME'],table='osm',user='postgres',password='gis')
    >>> params['estimate_extent'] = False
    >>> params['extent'] = '-20037508,-19929239,20037508,19929239'
    >>> postgis = PostGIS(**params)
    >>> lyr = Layer('PostGIS Layer')
    >>> lyr.datasource = postgis

    """
    keywords['type'] = 'postgis'
    return CreateDatasource(keywords)


def PgRaster(**keywords):
    """Create a PgRaster Datasource.

    Required keyword arguments:
      dbname -- database name to connect to
      table -- table name or subselect query

      *Note: if using subselects for the 'table' value consider also
       passing the 'raster_field' and 'srid' and 'extent_from_subquery'
       options and/or specifying the 'raster_table' option.

    Optional db connection keyword arguments:
      user -- database user to connect as (default: see postgres docs)
      password -- password for database user (default: see postgres docs)
      host -- postgres hostname (default: see postgres docs)
      port -- postgres port (default: see postgres docs)
      initial_size -- integer size of connection pool (default: 1)
      max_size -- integer max of connection pool (default: 10)
      persist_connection -- keep connection open (default: True)

    Optional table-level keyword arguments:
      extent -- manually specified data extent (comma delimited string, default: None)
      estimate_extent -- boolean, direct PostGIS to use the faster, less accurate `estimate_extent` over `extent` (default: False)
      extent_from_subquery -- boolean, direct Mapnik to query Postgis for the extent of the raw 'table' value (default: uses 'geometry_table')
      raster_table -- specify geometry table to use to look up metadata (default: automatically parsed from 'table' value)
      raster_field -- specify geometry field to use (default: first entry in raster_columns)
      srid -- specify srid to use (default: auto-detected from geometry_field)
      row_limit -- integer limit of rows to return (default: 0)
      cursor_size -- integer size of binary cursor to use (default: 0, no binary cursor is used)
      use_overviews -- boolean, use overviews when available (default: false)
      prescale_rasters -- boolean, scale rasters on the db side (default: false)
      clip_rasters -- boolean, clip rasters on the db side (default: false)
      band -- integer, if non-zero interprets the given band (1-based offset) as a data raster (default: 0)

    >>> from mapnik import PgRaster, Layer
    >>> params = dict(dbname='mapnik',table='osm',user='postgres',password='gis')
    >>> params['estimate_extent'] = False
    >>> params['extent'] = '-20037508,-19929239,20037508,19929239'
    >>> pgraster = PgRaster(**params)
    >>> lyr = Layer('PgRaster Layer')
    >>> lyr.datasource = pgraster

    """
    keywords['type'] = 'pgraster'
    return CreateDatasource(keywords)


def Raster(**keywords):
    """Create a Raster (Tiff) Datasource.

    Required keyword arguments:
      file -- path to stripped or tiled tiff
      lox -- lowest (min) x/longitude of tiff extent
      loy -- lowest (min) y/latitude of tiff extent
      hix -- highest (max) x/longitude of tiff extent
      hiy -- highest (max) y/latitude of tiff extent

    Hint: lox,loy,hix,hiy make a Mapnik Box2d

    Optional keyword arguments:
      base -- path prefix (default None)
      multi -- whether the image is in tiles on disk (default False)

    Multi-tiled keyword arguments:
      x_width -- virtual image number of tiles in X direction (required)
      y_width -- virtual image number of tiles in Y direction (required)
      tile_size -- if an image is in tiles, how large are the tiles (default 256)
      tile_stride -- if an image is in tiles, what's the increment between rows/cols (default 1)

    >>> from mapnik import Raster, Layer
    >>> raster = Raster(base='/home/mapnik/data',file='elevation.tif',lox=-122.8,loy=48.5,hix=-122.7,hiy=48.6)
    >>> lyr = Layer('Tiff Layer')
    >>> lyr.datasource = raster

    """
    keywords['type'] = 'raster'
    return CreateDatasource(keywords)


def Gdal(**keywords):
    """Create a GDAL Raster Datasource.

    Required keyword arguments:
      file -- path to GDAL supported dataset

    Optional keyword arguments:
      base -- path prefix (default None)
      shared -- boolean, open GdalDataset in shared mode (default: False)
      bbox -- tuple (minx, miny, maxx, maxy). If specified, overrides the bbox detected by GDAL.

    >>> from mapnik import Gdal, Layer
    >>> dataset = Gdal(base='/home/mapnik/data',file='elevation.tif')
    >>> lyr = Layer('GDAL Layer from TIFF file')
    >>> lyr.datasource = dataset

    """
    keywords['type'] = 'gdal'
    if 'bbox' in keywords:
        if isinstance(keywords['bbox'], (tuple, list)):
            keywords['bbox'] = ','.join([str(item)
                                         for item in keywords['bbox']])
    return CreateDatasource(keywords)


def Occi(**keywords):
    """Create a Oracle Spatial (10g) Vector Datasource.

    Required keyword arguments:
      user -- database user to connect as
      password -- password for database user
      host -- oracle host to connect to (does not refer to SID in tsnames.ora)
      table -- table name or subselect query

    Optional keyword arguments:
      initial_size -- integer size of connection pool (default 1)
      max_size -- integer max of connection pool (default 10)
      extent -- manually specified data extent (comma delimited string, default None)
      estimate_extent -- boolean, direct Oracle to use the faster, less accurate estimate_extent() over extent() (default False)
      encoding -- file encoding (default 'utf-8')
      geometry_field -- specify geometry field (default 'GEOLOC')
      use_spatial_index -- boolean, force the use of the spatial index (default True)

    >>> from mapnik import Occi, Layer
    >>> params = dict(host='myoracle',user='scott',password='tiger',table='test')
    >>> params['estimate_extent'] = False
    >>> params['extent'] = '-20037508,-19929239,20037508,19929239'
    >>> oracle = Occi(**params)
    >>> lyr = Layer('Oracle Spatial Layer')
    >>> lyr.datasource = oracle
    """
    keywords['type'] = 'occi'
    return CreateDatasource(keywords)


def Ogr(**keywords):
    """Create a OGR Vector Datasource.

    Required keyword arguments:
      file -- path to OGR supported dataset
      layer -- name of layer to use within datasource (optional if layer_by_index or layer_by_sql is used)

    Optional keyword arguments:
      layer_by_index -- choose layer by index number instead of by layer name or sql.
      layer_by_sql -- choose layer by sql query number instead of by layer name or index.
      base -- path prefix (default None)
      encoding -- file encoding (default 'utf-8')

    >>> from mapnik import Ogr, Layer
    >>> datasource = Ogr(base='/home/mapnik/data',file='rivers.geojson',layer='OGRGeoJSON')
    >>> lyr = Layer('OGR Layer from GeoJSON file')
    >>> lyr.datasource = datasource

    """
    keywords['type'] = 'ogr'
    return CreateDatasource(keywords)


def SQLite(**keywords):
    """Create a SQLite Datasource.

    Required keyword arguments:
      file -- path to SQLite database file
      table -- table name or subselect query

    Optional keyword arguments:
      base -- path prefix (default None)
      encoding -- file encoding (default 'utf-8')
      extent -- manually specified data extent (comma delimited string, default None)
      metadata -- name of auxiliary table containing record for table with xmin, ymin, xmax, ymax, and f_table_name
      geometry_field -- name of geometry field (default 'the_geom')
      key_field -- name of primary key field (default 'OGC_FID')
      row_offset -- specify a custom integer row offset (default 0)
      row_limit -- specify a custom integer row limit (default 0)
      wkb_format -- specify a wkb type of 'spatialite' (default None)
      use_spatial_index -- boolean, instruct sqlite plugin to use Rtree spatial index (default True)

    >>> from mapnik import SQLite, Layer
    >>> sqlite = SQLite(base='/home/mapnik/data',file='osm.db',table='osm',extent='-20037508,-19929239,20037508,19929239')
    >>> lyr = Layer('SQLite Layer')
    >>> lyr.datasource = sqlite

    """
    keywords['type'] = 'sqlite'
    return CreateDatasource(keywords)


def Rasterlite(**keywords):
    """Create a Rasterlite Datasource.

    Required keyword arguments:
      file -- path to Rasterlite database file
      table -- table name or subselect query

    Optional keyword arguments:
      base -- path prefix (default None)
      extent -- manually specified data extent (comma delimited string, default None)

    >>> from mapnik import Rasterlite, Layer
    >>> rasterlite = Rasterlite(base='/home/mapnik/data',file='osm.db',table='osm',extent='-20037508,-19929239,20037508,19929239')
    >>> lyr = Layer('Rasterlite Layer')
    >>> lyr.datasource = rasterlite

    """
    keywords['type'] = 'rasterlite'
    return CreateDatasource(keywords)


def Osm(**keywords):
    """Create a Osm Datasource.

    Required keyword arguments:
      file -- path to OSM file

    Optional keyword arguments:
      encoding -- file encoding (default 'utf-8')
      url -- url to fetch data (default None)
      bbox -- data bounding box for fetching data (default None)

    >>> from mapnik import Osm, Layer
    >>> datasource = Osm(file='test.osm')
    >>> lyr = Layer('Osm Layer')
    >>> lyr.datasource = datasource

    """
    # note: parser only supports libxml2 so not exposing option
    # parser -- xml parser to use (default libxml2)
    keywords['type'] = 'osm'
    return CreateDatasource(keywords)


def Python(**keywords):
    """Create a Python Datasource.

    >>> from mapnik import Python, PythonDatasource
    >>> datasource = Python('PythonDataSource')
    >>> lyr = Layer('Python datasource')
    >>> lyr.datasource = datasource
    """
    keywords['type'] = 'python'
    return CreateDatasource(keywords)


def MemoryDatasource(**keywords):
    """Create a Memory Datasource.

    Optional keyword arguments:
        (TODO)
    """
    params = Parameters()
    params.append(Parameter('type', 'memory'))
    return MemoryDatasourceBase(params)


class PythonDatasource(object):
    """A base class for a Python data source.

    Optional arguments:
      envelope -- a mapnik.Box2d (minx, miny, maxx, maxy) envelope of the data source, default (-180,-90,180,90)
      geometry_type -- one of the DataGeometryType enumeration values, default Point
      data_type -- one of the DataType enumerations, default Vector
    """

    def __init__(self, envelope=None, geometry_type=None, data_type=None):
        self.envelope = envelope or Box2d(-180, -90, 180, 90)
        self.geometry_type = geometry_type or DataGeometryType.Point
        self.data_type = data_type or DataType.Vector

    def features(self, query):
        """Return an iterable which yields instances of Feature for features within the passed query.

        Required arguments:
          query -- a Query instance specifying the region for which features should be returned
        """
        return None

    def features_at_point(self, point):
        """Rarely used. Return an iterable which yields instances of Feature for the specified point."""
        return None

    @classmethod
    def wkb_features(cls, keys, features):
        """A convenience function to wrap an iterator yielding pairs of WKB format geometry and dictionaries of
        key-value pairs into mapnik features. Return this from PythonDatasource.features() passing it a sequence of keys
        to appear in the output and an iterator yielding features.

        For example. One might have a features() method in a derived class like the following:

        def features(self, query):
            # ... create WKB features feat1 and feat2

            return mapnik.PythonDatasource.wkb_features(
                keys = ( 'name', 'author' ),
                features = [
                    (feat1, { 'name': 'feat1', 'author': 'alice' }),
                    (feat2, { 'name': 'feat2', 'author': 'bob' }),
                ]
            )

        """
        ctx = Context()
        [ctx.push(x) for x in keys]

        def make_it(feat, idx):
            f = Feature(ctx, idx)
            geom, attrs = feat
            f.add_geometries_from_wkb(geom)
            for k, v in attrs.iteritems():
                f[k] = v
            return f

        return itertools.imap(make_it, features, itertools.count(1))

    @classmethod
    def wkt_features(cls, keys, features):
        """A convenience function to wrap an iterator yielding pairs of WKT format geometry and dictionaries of
        key-value pairs into mapnik features. Return this from PythonDatasource.features() passing it a sequence of keys
        to appear in the output and an iterator yielding features.

        For example. One might have a features() method in a derived class like the following:

        def features(self, query):
            # ... create WKT features feat1 and feat2

            return mapnik.PythonDatasource.wkt_features(
                keys = ( 'name', 'author' ),
                features = [
                    (feat1, { 'name': 'feat1', 'author': 'alice' }),
                    (feat2, { 'name': 'feat2', 'author': 'bob' }),
                ]
            )

        """
        ctx = Context()
        [ctx.push(x) for x in keys]

        def make_it(feat, idx):
            f = Feature(ctx, idx)
            geom, attrs = feat
            f.add_geometries_from_wkt(geom)
            for k, v in attrs.iteritems():
                f[k] = v
            return f

        return itertools.imap(make_it, features, itertools.count(1))


class _TextSymbolizer(TextSymbolizer, _injector()):

    @property
    def name(self):
        if isinstance(self.properties.format_tree, FormattingText):
            return self.properties.format_tree.text
        else:
            # There is no single expression which could be returned as name
            raise RuntimeError(
                "TextSymbolizer uses complex formatting features, but old compatibility interface is used to access it. Use self.properties.format_tree instead.")

    @name.setter
    def name(self, name):
        self.properties.format_tree = FormattingText(name)

    @property
    def text_size(self):
        return self.format.text_size

    @text_size.setter
    def text_size(self, text_size):
        self.format.text_size = text_size

    @property
    def face_name(self):
        return self.format.face_name

    @face_name.setter
    def face_name(self, face_name):
        self.format.face_name = face_name

    @property
    def fontset(self):
        return self.format.fontset

    @fontset.setter
    def fontset(self, fontset):
        self.format.fontset = fontset

    @property
    def character_spacing(self):
        return self.format.character_spacing

    @character_spacing.setter
    def character_spacing(self, character_spacing):
        self.format.character_spacing = character_spacing

    @property
    def line_spacing(self):
        return self.format.line_spacing

    @line_spacing.setter
    def line_spacing(self, line_spacing):
        self.format.line_spacing = line_spacing

    @property
    def text_opacity(self):
        return self.format.text_opacity

    @text_opacity.setter
    def text_opacity(self, text_opacity):
        self.format.text_opacity = text_opacity

    @property
    def wrap_before(self):
        return self.format.wrap_before

    @wrap_before.setter
    def wrap_before(self, wrap_before):
        self.format.wrap_before = wrap_before

    @property
    def text_transform(self):
        return self.format.text_transform

    @text_transform.setter
    def text_transform(self, text_transform):
        self.format.text_transform = text_transform

    @property
    def fill(self):
        return self.format.fill

    @fill.setter
    def fill(self, fill):
        self.format.fill = fill

    @property
    def halo_fill(self):
        return self.format.halo_fill

    @halo_fill.setter
    def halo_fill(self, halo_fill):
        self.format.halo_fill = halo_fill

    @property
    def halo_radius(self):
        return self.format.halo_radius

    @halo_radius.setter
    def halo_radius(self, halo_radius):
        self.format.halo_radius = halo_radius

    @property
    def label_placement(self):
        return self.properties.label_placement

    @label_placement.setter
    def label_placement(self, label_placement):
        self.properties.label_placement = label_placement

    @property
    def horizontal_alignment(self):
        return self.properties.horizontal_alignment

    @horizontal_alignment.setter
    def horizontal_alignment(self, horizontal_alignment):
        self.properties.horizontal_alignment = horizontal_alignment

    @property
    def justify_alignment(self):
        return self.properties.justify_alignment

    @justify_alignment.setter
    def justify_alignment(self, justify_alignment):
        self.properties.justify_alignment = justify_alignment

    @property
    def vertical_alignment(self):
        return self.properties.vertical_alignment

    @vertical_alignment.setter
    def vertical_alignment(self, vertical_alignment):
        self.properties.vertical_alignment = vertical_alignment

    @property
    def orientation(self):
        return self.properties.orientation

    @orientation.setter
    def orientation(self, orientation):
        self.properties.orientation = orientation

    @property
    def displacement(self):
        return self.properties.displacement

    @displacement.setter
    def displacement(self, displacement):
        self.properties.displacement = displacement

    @property
    def label_spacing(self):
        return self.properties.label_spacing

    @label_spacing.setter
    def label_spacing(self, label_spacing):
        self.properties.label_spacing = label_spacing

    @property
    def label_position_tolerance(self):
        return self.properties.label_position_tolerance

    @label_position_tolerance.setter
    def label_position_tolerance(self, label_position_tolerance):
        self.properties.label_position_tolerance = label_position_tolerance

    @property
    def avoid_edges(self):
        return self.properties.avoid_edges

    @avoid_edges.setter
    def avoid_edges(self, avoid_edges):
        self.properties.avoid_edges = avoid_edges

    @property
    def minimum_distance(self):
        return self.properties.minimum_distance

    @minimum_distance.setter
    def minimum_distance(self, minimum_distance):
        self.properties.minimum_distance = minimum_distance

    @property
    def minimum_padding(self):
        return self.properties.minimum_padding

    @minimum_padding.setter
    def minimum_padding(self, minimum_padding):
        self.properties.minimum_padding = minimum_padding

    @property
    def minimum_path_length(self):
        return self.properties.minimum_path_length

    @minimum_path_length.setter
    def minimum_path_length(self, minimum_path_length):
        self.properties.minimum_path_length = minimum_path_length

    @property
    def maximum_angle_char_delta(self):
        return self.properties.maximum_angle_char_delta

    @maximum_angle_char_delta.setter
    def maximum_angle_char_delta(self, maximum_angle_char_delta):
        self.properties.maximum_angle_char_delta = maximum_angle_char_delta

    @property
    def allow_overlap(self):
        return self.properties.allow_overlap

    @allow_overlap.setter
    def allow_overlap(self, allow_overlap):
        self.properties.allow_overlap = allow_overlap

    @property
    def text_ratio(self):
        return self.properties.text_ratio

    @text_ratio.setter
    def text_ratio(self, text_ratio):
        self.properties.text_ratio = text_ratio

    @property
    def wrap_width(self):
        return self.properties.wrap_width

    @wrap_width.setter
    def wrap_width(self, wrap_width):
        self.properties.wrap_width = wrap_width


def mapnik_version_from_string(version_string):
    """Return the Mapnik version from a string."""
    n = version_string.split('.')
    return (int(n[0]) * 100000) + (int(n[1]) * 100) + (int(n[2]))


def register_plugins(path=None):
    """Register plugins located by specified path"""
    if not path:
        if 'MAPNIK_INPUT_PLUGINS_DIRECTORY' in os.environ:
            path = os.environ.get('MAPNIK_INPUT_PLUGINS_DIRECTORY')
        else:
            from .paths import inputpluginspath
            path = inputpluginspath
    DatasourceCache.register_datasources(path)


def register_fonts(path=None, valid_extensions=[
                   '.ttf', '.otf', '.ttc', '.pfa', '.pfb', '.ttc', '.dfont', '.woff']):
    """Recursively register fonts using path argument as base directory"""
    if not path:
        if 'MAPNIK_FONT_DIRECTORY' in os.environ:
            path = os.environ.get('MAPNIK_FONT_DIRECTORY')
        else:
            from .paths import fontscollectionpath
            path = fontscollectionpath
    for dirpath, _, filenames in os.walk(path):
        for filename in filenames:
            if os.path.splitext(filename.lower())[1] in valid_extensions:
                FontEngine.register_font(os.path.join(dirpath, filename))

# auto-register known plugins and fonts
register_plugins()
register_fonts()



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_multi_tile_policy():
    srs = '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
    lyr = mapnik.Layer('raster')
    if 'raster' in mapnik.DatasourceCache.plugin_names():
        lyr.datasource = mapnik.Raster(
            file='../data/raster_tiles/${x}/${y}.tif',
            lox=-180,
            loy=-90,
            hix=180,
            hiy=90,
            multi=1,
            tile_size=256,
            x_width=2,
            y_width=2
        )
        lyr.srs = srs
        _map = mapnik.Map(256, 256, srs)
        style = mapnik.Style()
        rule = mapnik.Rule()
        sym = mapnik.RasterSymbolizer()
        rule.symbols.append(sym)
        style.rules.append(rule)
        _map.append_style('foo', style)
        lyr.styles.append('foo')
        _map.layers.append(lyr)
        _map.zoom_to_box(lyr.envelope())

        im = mapnik.Image(_map.width, _map.height)
        mapnik.render(_map, im)

        # test green chunk
        eq_(im.view(0, 64, 1, 1).tostring(), b'\x00\xff\x00\xff')
        eq_(im.view(127, 64, 1, 1).tostring(), b'\x00\xff\x00\xff')
        eq_(im.view(0, 127, 1, 1).tostring(), b'\x00\xff\x00\xff')
        eq_(im.view(127, 127, 1, 1).tostring(), b'\x00\xff\x00\xff')

        # test blue chunk
        eq_(im.view(128, 64, 1, 1).tostring(), b'\x00\x00\xff\xff')
        eq_(im.view(255, 64, 1, 1).tostring(), b'\x00\x00\xff\xff')
        eq_(im.view(128, 127, 1, 1).tostring(), b'\x00\x00\xff\xff')
        eq_(im.view(255, 127, 1, 1).tostring(), b'\x00\x00\xff\xff')

        # test red chunk
        eq_(im.view(0, 128, 1, 1).tostring(), b'\xff\x00\x00\xff')
        eq_(im.view(127, 128, 1, 1).tostring(), b'\xff\x00\x00\xff')
        eq_(im.view(0, 191, 1, 1).tostring(), b'\xff\x00\x00\xff')
        eq_(im.view(127, 191, 1, 1).tostring(), b'\xff\x00\x00\xff')

        # test magenta chunk
        eq_(im.view(128, 128, 1, 1).tostring(), b'\xff\x00\xff\xff')
        eq_(im.view(255, 128, 1, 1).tostring(), b'\xff\x00\xff\xff')
        eq_(im.view(128, 191, 1, 1).tostring(), b'\xff\x00\xff\xff')
        eq_(im.view(255, 191, 1, 1).tostring(), b'\xff\x00\xff\xff')

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_arbitrary_parameters_attached_to_map():
    m = mapnik.Map(256, 256)
    mapnik.load_map(m, '../data/good_maps/extra_arbitary_map_parameters.xml')
    eq_(len(m.parameters), 5)
    eq_(m.parameters['key'], 'value2')
    eq_(m.parameters['key3'], 'value3')
    eq_(m.parameters['unicode'], u'iván')
    eq_(m.parameters['integer'], 10)
    eq_(m.parameters['decimal'], .999)
    m2 = mapnik.Map(256, 256)
    for k, v in m.parameters:
        m2.parameters.append(mapnik.Parameter(k, v))
    eq_(len(m2.parameters), 5)
    eq_(m2.parameters['key'], 'value2')
    eq_(m2.parameters['key3'], 'value3')
    eq_(m2.parameters['unicode'], u'iván')
    eq_(m2.parameters['integer'], 10)
    eq_(m2.parameters['decimal'], .999)
    map_string = mapnik.save_map_to_string(m)
    m3 = mapnik.Map(256, 256)
    mapnik.load_map_from_string(m3, map_string)
    eq_(len(m3.parameters), 5)
    eq_(m3.parameters['key'], 'value2')
    eq_(m3.parameters['key3'], 'value3')
    eq_(m3.parameters['unicode'], u'iván')
    eq_(m3.parameters['integer'], 10)
    eq_(m3.parameters['decimal'], .999)


def test_serializing_arbitrary_parameters():
    m = mapnik.Map(256, 256)
    m.parameters.append(mapnik.Parameter('width', m.width))
    m.parameters.append(mapnik.Parameter('height', m.height))

    m2 = mapnik.Map(1, 1)
    mapnik.load_map_from_string(m2, mapnik.save_map_to_string(m))
    eq_(m2.parameters['width'], m.width)
    eq_(m2.parameters['height'], m.height)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_another_compare():
    im = mapnik.Image(5, 5)
    im2 = mapnik.Image(5, 5)
    im2.fill(mapnik.Color('rgba(255,255,255,0)'))
    eq_(im.compare(im2, 16), im.width() * im.height())


def test_compare_rgba8():
    im = mapnik.Image(5, 5, mapnik.ImageType.rgba8)
    im.fill(mapnik.Color(0, 0, 0, 0))
    eq_(im.compare(im), 0)
    im2 = mapnik.Image(5, 5, mapnik.ImageType.rgba8)
    im2.fill(mapnik.Color(0, 0, 0, 0))
    eq_(im.compare(im2), 0)
    eq_(im2.compare(im), 0)
    im2.fill(mapnik.Color(0, 0, 0, 12))
    eq_(im.compare(im2), 25)
    eq_(im.compare(im2, 0, False), 0)
    im3 = mapnik.Image(5, 5, mapnik.ImageType.rgba8)
    im3.set_pixel(0, 0, mapnik.Color(0, 0, 0, 0))
    im3.set_pixel(0, 1, mapnik.Color(1, 1, 1, 1))
    im3.set_pixel(1, 0, mapnik.Color(2, 2, 2, 2))
    im3.set_pixel(1, 1, mapnik.Color(3, 3, 3, 3))
    eq_(im.compare(im3), 3)
    eq_(im.compare(im3, 1), 2)
    eq_(im.compare(im3, 2), 1)
    eq_(im.compare(im3, 3), 0)


def test_compare_2_image():
    im = mapnik.Image(5, 5)
    im.set_pixel(0, 0, mapnik.Color(254, 254, 254, 254))
    im.set_pixel(4, 4, mapnik.Color('white'))
    im2 = mapnik.Image(5, 5)
    eq_(im2.compare(im, 16), 2)


def test_compare_dimensions():
    im = mapnik.Image(2, 2)
    im2 = mapnik.Image(3, 3)
    eq_(im.compare(im2), 4)
    eq_(im2.compare(im), 9)


def test_compare_gray8():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray8)
    im.fill(0)
    eq_(im.compare(im), 0)
    im2 = mapnik.Image(2, 2, mapnik.ImageType.gray8)
    im2.fill(0)
    eq_(im.compare(im2), 0)
    eq_(im2.compare(im), 0)
    eq_(im.compare(im2, 0, False), 0)
    im3 = mapnik.Image(2, 2, mapnik.ImageType.gray8)
    im3.set_pixel(0, 0, 0)
    im3.set_pixel(0, 1, 1)
    im3.set_pixel(1, 0, 2)
    im3.set_pixel(1, 1, 3)
    eq_(im.compare(im3), 3)
    eq_(im.compare(im3, 1), 2)
    eq_(im.compare(im3, 2), 1)
    eq_(im.compare(im3, 3), 0)


def test_compare_gray16():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im.fill(0)
    eq_(im.compare(im), 0)
    im2 = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im2.fill(0)
    eq_(im.compare(im2), 0)
    eq_(im2.compare(im), 0)
    eq_(im.compare(im2, 0, False), 0)
    im3 = mapnik.Image(2, 2, mapnik.ImageType.gray16)
    im3.set_pixel(0, 0, 0)
    im3.set_pixel(0, 1, 1)
    im3.set_pixel(1, 0, 2)
    im3.set_pixel(1, 1, 3)
    eq_(im.compare(im3), 3)
    eq_(im.compare(im3, 1), 2)
    eq_(im.compare(im3, 2), 1)
    eq_(im.compare(im3, 3), 0)


def test_compare_gray32f():
    im = mapnik.Image(2, 2, mapnik.ImageType.gray32f)
    im.fill(0.5)
    eq_(im.compare(im), 0)
    im2 = mapnik.Image(2, 2, mapnik.ImageType.gray32f)
    im2.fill(0.5)
    eq_(im.compare(im2), 0)
    eq_(im2.compare(im), 0)
    eq_(im.compare(im2, 0, False), 0)
    im3 = mapnik.Image(2, 2, mapnik.ImageType.gray32f)
    im3.set_pixel(0, 0, 0.5)
    im3.set_pixel(0, 1, 1.5)
    im3.set_pixel(1, 0, 2.5)
    im3.set_pixel(1, 1, 3.5)
    eq_(im.compare(im3), 3)
    eq_(im.compare(im3, 1.0), 2)
    eq_(im.compare(im3, 2.0), 1)
    eq_(im.compare(im3, 3.0), 0)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_adding_datasource_to_layer():
    map_string = '''<?xml version="1.0" encoding="utf-8"?>
<Map>

    <Layer name="world_borders">
        <StyleName>world_borders_style</StyleName>
        <StyleName>point_style</StyleName>
        <!-- leave datasource empty -->
        <!--
        <Datasource>
            <Parameter name="file">../data/shp/world_merc.shp</Parameter>
            <Parameter name="type">shape</Parameter>
        </Datasource>
        -->
    </Layer>

</Map>
'''
    m = mapnik.Map(256, 256)

    try:
        mapnik.load_map_from_string(m, map_string)

        # validate it loaded fine
        eq_(m.layers[0].styles[0], 'world_borders_style')
        eq_(m.layers[0].styles[1], 'point_style')
        eq_(len(m.layers), 1)

        # also assign a variable reference to that layer
        # below we will test that this variable references
        # the same object that is attached to the map
        lyr = m.layers[0]

        # ensure that there was no datasource for the layer...
        eq_(m.layers[0].datasource, None)
        eq_(lyr.datasource, None)

        # also note that since the srs was black it defaulted to wgs84
        eq_(m.layers[0].srs,
            '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')
        eq_(lyr.srs, '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')

        # now add a datasource one...
        ds = mapnik.Shapefile(file='../data/shp/world_merc.shp')
        m.layers[0].datasource = ds

        # now ensure it is attached
        eq_(m.layers[0].datasource.describe()['name'], "shape")
        eq_(lyr.datasource.describe()['name'], "shape")

        # and since we have now added a shapefile in spherical mercator, adjust
        # the projection
        lyr.srs = '+proj=merc +lon_0=0 +lat_ts=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs'

        # test that assignment
        eq_(m.layers[
            0].srs, '+proj=merc +lon_0=0 +lat_ts=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs')
        eq_(lyr.srs, '+proj=merc +lon_0=0 +lat_ts=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs')
    except RuntimeError as e:
        # only test datasources that we have installed
        if not 'Could not create datasource' in str(e):
            raise RuntimeError(e)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_datasource_template_is_working():
    m = mapnik.Map(256, 256)
    try:
        mapnik.load_map(m, '../data/good_maps/datasource.xml')
    except RuntimeError as e:
        if "Required parameter 'type'" in str(e):
            raise RuntimeError(e)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import glob
import os

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path


default_logging_severity = mapnik.logger.get_severity()


def setup():
    # make the tests silent since we intentially test error conditions that
    # are noisy
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def teardown():
    mapnik.logger.set_severity(default_logging_severity)

if 'csv' in mapnik.DatasourceCache.plugin_names():

    def get_csv_ds(filename):
        return mapnik.Datasource(
            type='csv', file=os.path.join('../data/csv/', filename))

    def test_broken_files(visual=False):
        broken = glob.glob("../data/csv/fails/*.*")
        broken.extend(glob.glob("../data/csv/warns/*.*"))

        # Add a filename that doesn't exist
        broken.append("../data/csv/fails/does_not_exist.csv")

        for csv in broken:
            if visual:
                try:
                    mapnik.Datasource(type='csv', file=csv, strict=True)
                    print('\x1b[33mfailed: should have thrown\x1b[0m', csv)
                except Exception:
                    print('\x1b[1;32m✓ \x1b[0m', csv)

    def test_good_files(visual=False):
        good_files = glob.glob("../data/csv/*.*")
        good_files.extend(glob.glob("../data/csv/warns/*.*"))
        ignorable = os.path.join('..', 'data', 'csv', 'long_lat.vrt')
        good_files.remove(ignorable)
        for f in good_files:
            if f.endswith('.index'):
                good_files.remove(f)
        for csv in good_files:
            if visual:
                try:
                    mapnik.Datasource(type='csv', file=csv)
                    print('\x1b[1;32m✓ \x1b[0m', csv)
                except Exception as e:
                    print(
                        '\x1b[33mfailed: should not have thrown\x1b[0m',
                        csv,
                        str(e))

    def test_lon_lat_detection(**kwargs):
        ds = get_csv_ds('lon_lat.csv')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['lon', 'lat'])
        eq_(ds.field_types(), ['int', 'int'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        fs = ds.features(query)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        feat = fs.next()
        attr = {'lon': 0, 'lat': 0}
        eq_(feat.attributes, attr)

    def test_lng_lat_detection(**kwargs):
        ds = get_csv_ds('lng_lat.csv')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['lng', 'lat'])
        eq_(ds.field_types(), ['int', 'int'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        fs = ds.features(query)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        feat = fs.next()
        attr = {'lng': 0, 'lat': 0}
        eq_(feat.attributes, attr)

    def test_type_detection(**kwargs):
        ds = get_csv_ds('nypd.csv')
        eq_(ds.fields(),
            ['Precinct',
             'Phone',
             'Address',
             'City',
             'geo_longitude',
             'geo_latitude',
             'geo_accuracy'])
        eq_(ds.field_types(), ['str', 'str',
                               'str', 'str', 'float', 'float', 'str'])
        feat = ds.featureset().next()
        attr = {
            'City': u'New York, NY',
            'geo_accuracy': u'house',
            'Phone': u'(212) 334-0711',
            'Address': u'19 Elizabeth Street',
            'Precinct': u'5th Precinct',
            'geo_longitude': -70,
            'geo_latitude': 40}
        eq_(feat.attributes, attr)
        eq_(len(list(ds.all_features())), 2)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_skipping_blank_rows(**kwargs):
        ds = get_csv_ds('blank_rows.csv')
        eq_(ds.fields(), ['x', 'y', 'name'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        eq_(len(list(ds.all_features())), 2)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_empty_rows(**kwargs):
        ds = get_csv_ds('empty_rows.csv')
        eq_(len(ds.fields()), 10)
        eq_(len(ds.field_types()), 10)
        eq_(ds.fields(), ['x', 'y', 'text', 'date', 'integer',
                          'boolean', 'float', 'time', 'datetime', 'empty_column'])
        eq_(ds.field_types(), ['int', 'int', 'str', 'str',
                               'int', 'bool', 'float', 'str', 'str', 'str'])
        fs = ds.featureset()
        attr = {
            'x': 0,
            'empty_column': u'',
            'text': u'a b',
            'float': 1.0,
            'datetime': u'1971-01-01T04:14:00',
            'y': 0,
            'boolean': True,
            'time': u'04:14:00',
            'date': u'1971-01-01',
            'integer': 40}
        first = True
        for feat in fs:
            if first:
                first = False
                eq_(feat.attributes, attr)
            eq_(len(feat), 10)
            eq_(feat['empty_column'], u'')

        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_slashes(**kwargs):
        ds = get_csv_ds('has_attributes_with_slashes.csv')
        eq_(len(ds.fields()), 3)
        fs = list(ds.all_features())
        eq_(fs[0].attributes, {'x': 0, 'y': 0, 'name': u'a/a'})
        eq_(fs[1].attributes, {'x': 1, 'y': 4, 'name': u'b/b'})
        eq_(fs[2].attributes, {'x': 10, 'y': 2.5, 'name': u'c/c'})
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_wkt_field(**kwargs):
        ds = get_csv_ds('wkt.csv')
        eq_(len(ds.fields()), 1)
        eq_(ds.fields(), ['type'])
        eq_(ds.field_types(), ['str'])
        fs = list(ds.all_features())
        # eq_(len(fs[0].geometries()),1)
        eq_(fs[0].geometry.type(), mapnik.GeometryType.Point)
        # eq_(len(fs[1].geometries()),1)
        eq_(fs[1].geometry.type(), mapnik.GeometryType.LineString)
        # eq_(len(fs[2].geometries()),1)
        eq_(fs[2].geometry.type(), mapnik.GeometryType.Polygon)
        # eq_(len(fs[3].geometries()),1) # one geometry, two parts
        eq_(fs[3].geometry.type(), mapnik.GeometryType.Polygon)
        # eq_(len(fs[4].geometries()),4)
        eq_(fs[4].geometry.type(), mapnik.GeometryType.MultiPoint)
        # eq_(len(fs[5].geometries()),2)
        eq_(fs[5].geometry.type(), mapnik.GeometryType.MultiLineString)
        # eq_(len(fs[6].geometries()),2)
        eq_(fs[6].geometry.type(), mapnik.GeometryType.MultiPolygon)
        # eq_(len(fs[7].geometries()),2)
        eq_(fs[7].geometry.type(), mapnik.GeometryType.MultiPolygon)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Collection)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_handling_of_missing_header(**kwargs):
        ds = get_csv_ds('missing_header.csv')
        eq_(len(ds.fields()), 6)
        eq_(ds.fields(), ['one', 'two', 'x', 'y', '_4', 'aftermissing'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['_4'], 'missing')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_handling_of_headers_that_are_numbers(**kwargs):
        ds = get_csv_ds('numbers_for_headers.csv')
        eq_(len(ds.fields()), 5)
        eq_(ds.fields(), ['x', 'y', '1990', '1991', '1992'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['1990'], 1)
        eq_(feat['1991'], 2)
        eq_(feat['1992'], 3)
        eq_(mapnik.Expression("[1991]=2").evaluate(feat), True)

    def test_quoted_numbers(**kwargs):
        ds = get_csv_ds('points.csv')
        eq_(len(ds.fields()), 6)
        eq_(ds.fields(), ['lat', 'long', 'name', 'nr', 'color', 'placements'])
        fs = list(ds.all_features())
        eq_(fs[0]['placements'], "N,S,E,W,SW,10,5")
        eq_(fs[1]['placements'], "N,S,E,W,SW,10,5")
        eq_(fs[2]['placements'], "N,S,E,W,SW,10,5")
        eq_(fs[3]['placements'], "N,S,E,W,SW,10,5")
        eq_(fs[4]['placements'], "N,S,E,W,SW,10,5")
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_reading_windows_newlines(**kwargs):
        ds = get_csv_ds('windows_newlines.csv')
        eq_(len(ds.fields()), 3)
        feats = list(ds.all_features())
        eq_(len(feats), 1)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 1)
        eq_(feat['y'], 10)
        eq_(feat['z'], 9999.9999)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_reading_mac_newlines(**kwargs):
        ds = get_csv_ds('mac_newlines.csv')
        eq_(len(ds.fields()), 3)
        feats = list(ds.all_features())
        eq_(len(feats), 1)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 1)
        eq_(feat['y'], 10)
        eq_(feat['z'], 9999.9999)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def check_newlines(filename):
        ds = get_csv_ds(filename)
        eq_(len(ds.fields()), 3)
        feats = list(ds.all_features())
        eq_(len(feats), 1)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['line'], 'many\n  lines\n  of text\n  with unix newlines')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_mixed_mac_unix_newlines(**kwargs):
        check_newlines('mac_newlines_with_unix_inline.csv')

    def test_mixed_mac_unix_newlines_escaped(**kwargs):
        check_newlines('mac_newlines_with_unix_inline_escaped.csv')

    # To hard to support this case
    # def test_mixed_unix_windows_newlines(**kwargs):
    #    check_newlines('unix_newlines_with_windows_inline.csv')

    # To hard to support this case
    # def test_mixed_unix_windows_newlines_escaped(**kwargs):
    #    check_newlines('unix_newlines_with_windows_inline_escaped.csv')

    def test_mixed_windows_unix_newlines(**kwargs):
        check_newlines('windows_newlines_with_unix_inline.csv')

    def test_mixed_windows_unix_newlines_escaped(**kwargs):
        check_newlines('windows_newlines_with_unix_inline_escaped.csv')

    def test_tabs(**kwargs):
        ds = get_csv_ds('tabs_in_csv.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'z'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], -122)
        eq_(feat['y'], 48)
        eq_(feat['z'], 0)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_separator_pipes(**kwargs):
        ds = get_csv_ds('pipe_delimiters.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'z'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['z'], 'hello')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_separator_semicolon(**kwargs):
        ds = get_csv_ds('semicolon_delimiters.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'z'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['z'], 'hello')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_that_null_and_bool_keywords_are_empty_strings(**kwargs):
        ds = get_csv_ds('nulls_and_booleans_as_strings.csv')
        eq_(len(ds.fields()), 4)
        eq_(ds.fields(), ['x', 'y', 'null', 'boolean'])
        eq_(ds.field_types(), ['int', 'int', 'str', 'bool'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['null'], 'null')
        eq_(feat['boolean'], True)
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['null'], '')
        eq_(feat['boolean'], False)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)

    @raises(RuntimeError)
    def test_that_nonexistant_query_field_throws(**kwargs):
        ds = get_csv_ds('lon_lat.csv')
        eq_(len(ds.fields()), 2)
        eq_(ds.fields(), ['lon', 'lat'])
        eq_(ds.field_types(), ['int', 'int'])
        query = mapnik.Query(ds.envelope())
        for fld in ds.fields():
            query.add_property_name(fld)
        # also add an invalid one, triggering throw
        query.add_property_name('bogus')
        ds.features(query)

    def test_that_leading_zeros_mean_strings(**kwargs):
        ds = get_csv_ds('leading_zeros.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'fips'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['fips'], '001')
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['fips'], '003')
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['fips'], '005')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)

    def test_advanced_geometry_detection(**kwargs):
        ds = get_csv_ds('point_wkt.csv')
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.Point)
        ds = get_csv_ds('poly_wkt.csv')
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.Polygon)
        ds = get_csv_ds('multi_poly_wkt.csv')
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.Polygon)
        ds = get_csv_ds('line_wkt.csv')
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.LineString)

    def test_creation_of_csv_from_in_memory_string(**kwargs):
        csv_string = '''
           wkt,Name
          "POINT (120.15 48.47)","Winthrop, WA"
          '''  # csv plugin will test lines <= 10 chars for being fully blank
        ds = mapnik.Datasource(**{"type": "csv", "inline": csv_string})
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.Point)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['Name'], u"Winthrop, WA")

    def test_creation_of_csv_from_in_memory_string_with_uft8(**kwargs):
        csv_string = '''
           wkt,Name
          "POINT (120.15 48.47)","Québec"
          '''  # csv plugin will test lines <= 10 chars for being fully blank
        ds = mapnik.Datasource(**{"type": "csv", "inline": csv_string})
        eq_(ds.describe()['geometry_type'], mapnik.DataGeometryType.Point)
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['Name'], u"Québec")

    def validate_geojson_datasource(ds):
        eq_(len(ds.fields()), 1)
        eq_(ds.fields(), ['type'])
        eq_(ds.field_types(), ['str'])
        fs = list(ds.all_features())
        # eq_(len(fs[0].geometries()),1)
        eq_(fs[0].geometry.type(), mapnik.GeometryType.Point)
        # eq_(len(fs[1].geometries()),1)
        eq_(fs[1].geometry.type(), mapnik.GeometryType.LineString)
        # eq_(len(fs[2].geometries()),1)
        eq_(fs[2].geometry.type(), mapnik.GeometryType.Polygon)
        # eq_(len(fs[3].geometries()),1) # one geometry, two parts
        eq_(fs[3].geometry.type(), mapnik.GeometryType.Polygon)
        # eq_(len(fs[4].geometries()),4)
        eq_(fs[4].geometry.type(), mapnik.GeometryType.MultiPoint)
        # eq_(len(fs[5].geometries()),2)
        eq_(fs[5].geometry.type(), mapnik.GeometryType.MultiLineString)
        # eq_(len(fs[6].geometries()),2)
        eq_(fs[6].geometry.type(), mapnik.GeometryType.MultiPolygon)
        # eq_(len(fs[7].geometries()),2)
        eq_(fs[7].geometry.type(), mapnik.GeometryType.MultiPolygon)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Collection)
        eq_(desc['name'], 'csv')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')

    def test_json_field1(**kwargs):
        ds = get_csv_ds('geojson_double_quote_escape.csv')
        validate_geojson_datasource(ds)

    def test_json_field2(**kwargs):
        ds = get_csv_ds('geojson_single_quote.csv')
        validate_geojson_datasource(ds)

    def test_json_field3(**kwargs):
        ds = get_csv_ds('geojson_2x_double_quote_filebakery_style.csv')
        validate_geojson_datasource(ds)

    def test_that_blank_undelimited_rows_are_still_parsed(**kwargs):
        ds = get_csv_ds('more_headers_than_column_values.csv')
        eq_(len(ds.fields()), 0)
        eq_(ds.fields(), [])
        eq_(ds.field_types(), [])
        fs = list(ds.featureset())
        eq_(len(fs), 0)
        desc = ds.describe()
        eq_(desc['geometry_type'], None)

    @raises(RuntimeError)
    def test_that_fewer_headers_than_rows_throws(**kwargs):
        # this has invalid header # so throw
        get_csv_ds('more_column_values_than_headers.csv')

    def test_that_feature_id_only_incremented_for_valid_rows(**kwargs):
        ds = mapnik.Datasource(type='csv',
                               file=os.path.join('../data/csv/warns', 'feature_id_counting.csv'))
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'id'])
        eq_(ds.field_types(), ['int', 'int', 'int'])
        fs = ds.featureset()
        # first
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['id'], 1)
        # second, should have skipped bogus one
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['id'], 2)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 2)

    def test_dynamically_defining_headers1(**kwargs):
        ds = mapnik.Datasource(type='csv',
                               file=os.path.join(
                                   '../data/csv/fails', 'needs_headers_two_lines.csv'),
                               headers='x,y,name')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'name'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['name'], 'data_name')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 2)

    def test_dynamically_defining_headers2(**kwargs):
        ds = mapnik.Datasource(type='csv',
                               file=os.path.join(
                                   '../data/csv/fails', 'needs_headers_one_line.csv'),
                               headers='x,y,name')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'name'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['name'], 'data_name')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 1)

    def test_dynamically_defining_headers3(**kwargs):
        ds = mapnik.Datasource(type='csv',
                               file=os.path.join(
                                   '../data/csv/fails', 'needs_headers_one_line_no_newline.csv'),
                               headers='x,y,name')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'name'])
        eq_(ds.field_types(), ['int', 'int', 'str'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['x'], 0)
        eq_(feat['y'], 0)
        eq_(feat['name'], 'data_name')
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 1)

    def test_that_64bit_int_fields_work(**kwargs):
        ds = get_csv_ds('64bit_int.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'bigint'])
        eq_(ds.field_types(), ['int', 'int', 'int'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['bigint'], 2147483648)
        feat = fs.next()
        eq_(feat['bigint'], 9223372036854775807)
        eq_(feat['bigint'], 0x7FFFFFFFFFFFFFFF)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 2)

    def test_various_number_types(**kwargs):
        ds = get_csv_ds('number_types.csv')
        eq_(len(ds.fields()), 3)
        eq_(ds.fields(), ['x', 'y', 'floats'])
        eq_(ds.field_types(), ['int', 'int', 'float'])
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat['floats'], .0)
        feat = fs.next()
        eq_(feat['floats'], +.0)
        feat = fs.next()
        eq_(feat['floats'], 1e-06)
        feat = fs.next()
        eq_(feat['floats'], -1e-06)
        feat = fs.next()
        eq_(feat['floats'], 0.000001)
        feat = fs.next()
        eq_(feat['floats'], 1.234e+16)
        feat = fs.next()
        eq_(feat['floats'], 1.234e+16)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(len(list(ds.all_features())), 8)

    def test_manually_supplied_extent(**kwargs):
        csv_string = '''
           wkt,Name
          '''
        ds = mapnik.Datasource(
            **{"type": "csv", "extent": "-180,-90,180,90", "inline": csv_string})
        b = ds.envelope()
        eq_(b.minx, -180)
        eq_(b.miny, -90)
        eq_(b.maxx, 180)
        eq_(b.maxy, 90)

    def test_inline_geojson(**kwargs):
        csv_string = "geojson\n'{\"coordinates\":[-92.22568,38.59553],\"type\":\"Point\"}'"
        ds = mapnik.Datasource(**{"type": "csv", "inline": csv_string})
        eq_(len(ds.fields()), 0)
        eq_(ds.fields(), [])
        # FIXME - re-enable after https://github.com/mapnik/mapnik/issues/2319 is fixed
        #fs = ds.featureset()
        #feat = fs.next()
        # eq_(feat.num_geometries(),1)

if __name__ == "__main__":
    setup()
    [eval(run)(visual=True) for run in dir() if 'test_' in run]



#!/usr/bin/env python

import glob
import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


default_logging_severity = mapnik.logger.get_severity()


def setup():
    # make the tests silent to suppress unsupported params from harfbuzz tests
    # TODO: remove this after harfbuzz branch merges
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def teardown():
    mapnik.logger.set_severity(default_logging_severity)


def test_broken_files():
    default_logging_severity = mapnik.logger.get_severity()
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    broken_files = glob.glob("../data/broken_maps/*.xml")
    # Add a filename that doesn't exist
    broken_files.append("../data/broken/does_not_exist.xml")

    failures = []
    for filename in broken_files:
        try:
            m = mapnik.Map(512, 512)
            strict = True
            mapnik.load_map(m, filename, strict)
            failures.append(
                'Loading broken map (%s) did not raise RuntimeError!' %
                filename)
        except RuntimeError:
            pass
    eq_(len(failures), 0, '\n' + '\n'.join(failures))
    mapnik.logger.set_severity(default_logging_severity)


def test_can_parse_xml_with_deprecated_properties():
    default_logging_severity = mapnik.logger.get_severity()
    mapnik.logger.set_severity(getattr(mapnik.severity_type, "None"))
    files_with_deprecated_props = glob.glob("../data/deprecated_maps/*.xml")

    failures = []
    for filename in files_with_deprecated_props:
        try:
            m = mapnik.Map(512, 512)
            strict = True
            mapnik.load_map(m, filename, strict)
            base_path = os.path.dirname(filename)
            mapnik.load_map_from_string(
                m,
                open(
                    filename,
                    'rb').read(),
                strict,
                base_path)
        except RuntimeError as e:
            # only test datasources that we have installed
            if not 'Could not create datasource' in str(e) \
               and not 'could not connect' in str(e):
                failures.append(
                    'Failed to load valid map %s (%s)' %
                    (filename, e))
    eq_(len(failures), 0, '\n' + '\n'.join(failures))
    mapnik.logger.set_severity(default_logging_severity)


def test_good_files():
    good_files = glob.glob("../data/good_maps/*.xml")
    good_files.extend(glob.glob("../visual_tests/styles/*.xml"))

    failures = []
    for filename in good_files:
        try:
            m = mapnik.Map(512, 512)
            strict = True
            mapnik.load_map(m, filename, strict)
            base_path = os.path.dirname(filename)
            with open(filename, 'rb') as f:
                mapnik.load_map_from_string(m, f.read(), strict, base_path)
        except RuntimeError as e:
            # only test datasources that we have installed
            if not 'Could not create datasource' in str(e) \
               and not 'could not connect' in str(e):
                failures.append(
                    'Failed to load valid map %s (%s)' %
                    (filename, e))
    eq_(len(failures), 0, '\n' + '\n'.join(failures))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



"""Unit conversion helpers."""

def m2pt(x, pt_size=0.0254/72.0):
    """Converts distance from meters to points. Default value is PDF point size."""
    return x / pt_size

def pt2m(x, pt_size=0.0254/72.0):
    """Converts distance from points to meters. Default value is PDF point size."""
    return x * pt_size

def m2in(x):
    """Converts distance from meters to inches."""
    return x / 0.0254

def m2px(x, resolution):
    """Converts distance from meters to pixels at the given resolution in DPI/PPI."""
    return m2in(x) * resolution



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_color_init():
    c = mapnik.Color(12, 128, 255)
    eq_(c.r, 12)
    eq_(c.g, 128)
    eq_(c.b, 255)
    eq_(c.a, 255)
    eq_(False, c.get_premultiplied())
    c = mapnik.Color(16, 32, 64, 128)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(False, c.get_premultiplied())
    c = mapnik.Color(16, 32, 64, 128, True)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(True, c.get_premultiplied())
    c = mapnik.Color('rgba(16,32,64,0.5)')
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(False, c.get_premultiplied())
    c = mapnik.Color('rgba(16,32,64,0.5)', True)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(True, c.get_premultiplied())
    hex_str = '#10204080'
    c = mapnik.Color(hex_str)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(hex_str, c.to_hex_string())
    eq_(False, c.get_premultiplied())
    c = mapnik.Color(hex_str, True)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(hex_str, c.to_hex_string())
    eq_(True, c.get_premultiplied())
    rgba_int = 2151686160
    c = mapnik.Color(rgba_int)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(rgba_int, c.packed())
    eq_(False, c.get_premultiplied())
    c = mapnik.Color(rgba_int, True)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    eq_(rgba_int, c.packed())
    eq_(True, c.get_premultiplied())


def test_color_properties():
    c = mapnik.Color(16, 32, 64, 128)
    eq_(c.r, 16)
    eq_(c.g, 32)
    eq_(c.b, 64)
    eq_(c.a, 128)
    c.r = 17
    eq_(c.r, 17)
    c.g = 33
    eq_(c.g, 33)
    c.b = 65
    eq_(c.b, 65)
    c.a = 128
    eq_(c.a, 128)


def test_color_premultiply():
    c = mapnik.Color(16, 33, 255, 128)
    eq_(c.premultiply(), True)
    eq_(c.r, 8)
    eq_(c.g, 17)
    eq_(c.b, 128)
    eq_(c.a, 128)
    # Repeating it again should do nothing
    eq_(c.premultiply(), False)
    eq_(c.r, 8)
    eq_(c.g, 17)
    eq_(c.b, 128)
    eq_(c.a, 128)
    c.demultiply()
    c.demultiply()
    # This will not return the same values as before but we expect that
    eq_(c.r, 15)
    eq_(c.g, 33)
    eq_(c.b, 255)
    eq_(c.a, 128)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import assert_almost_equal, assert_false, assert_true, eq_

import mapnik

from .utilities import run_all


def test_coord_init():
    c = mapnik.Coord(100, 100)

    eq_(c.x, 100)
    eq_(c.y, 100)


def test_coord_multiplication():
    c = mapnik.Coord(100, 100)
    c *= 2

    eq_(c.x, 200)
    eq_(c.y, 200)


def test_envelope_init():
    e = mapnik.Box2d(100, 100, 200, 200)

    assert_true(e.contains(100, 100))
    assert_true(e.contains(100, 200))
    assert_true(e.contains(200, 200))
    assert_true(e.contains(200, 100))

    assert_true(e.contains(e.center()))

    assert_false(e.contains(99.9, 99.9))
    assert_false(e.contains(99.9, 200.1))
    assert_false(e.contains(200.1, 200.1))
    assert_false(e.contains(200.1, 99.9))

    eq_(e.width(), 100)
    eq_(e.height(), 100)

    eq_(e.minx, 100)
    eq_(e.miny, 100)

    eq_(e.maxx, 200)
    eq_(e.maxy, 200)

    eq_(e[0], 100)
    eq_(e[1], 100)
    eq_(e[2], 200)
    eq_(e[3], 200)
    eq_(e[0], e[-4])
    eq_(e[1], e[-3])
    eq_(e[2], e[-2])
    eq_(e[3], e[-1])

    c = e.center()

    eq_(c.x, 150)
    eq_(c.y, 150)


def test_envelope_static_init():
    e = mapnik.Box2d.from_string('100 100 200 200')
    e2 = mapnik.Box2d.from_string('100,100,200,200')
    e3 = mapnik.Box2d.from_string('100 , 100 , 200 , 200')
    eq_(e, e2)
    eq_(e, e3)

    assert_true(e.contains(100, 100))
    assert_true(e.contains(100, 200))
    assert_true(e.contains(200, 200))
    assert_true(e.contains(200, 100))

    assert_true(e.contains(e.center()))

    assert_false(e.contains(99.9, 99.9))
    assert_false(e.contains(99.9, 200.1))
    assert_false(e.contains(200.1, 200.1))
    assert_false(e.contains(200.1, 99.9))

    eq_(e.width(), 100)
    eq_(e.height(), 100)

    eq_(e.minx, 100)
    eq_(e.miny, 100)

    eq_(e.maxx, 200)
    eq_(e.maxy, 200)

    eq_(e[0], 100)
    eq_(e[1], 100)
    eq_(e[2], 200)
    eq_(e[3], 200)
    eq_(e[0], e[-4])
    eq_(e[1], e[-3])
    eq_(e[2], e[-2])
    eq_(e[3], e[-1])

    c = e.center()

    eq_(c.x, 150)
    eq_(c.y, 150)


def test_envelope_multiplication():
    # no width then no impact of multiplication
    a = mapnik.Box2d(100, 100, 100, 100)
    a *= 5
    eq_(a.minx, 100)
    eq_(a.miny, 100)
    eq_(a.maxx, 100)
    eq_(a.maxy, 100)

    a = mapnik.Box2d(100.0, 100.0, 100.0, 100.0)
    a *= 5
    eq_(a.minx, 100)
    eq_(a.miny, 100)
    eq_(a.maxx, 100)
    eq_(a.maxy, 100)

    a = mapnik.Box2d(100.0, 100.0, 100.001, 100.001)
    a *= 5
    assert_almost_equal(a.minx, 99.9979, places=3)
    assert_almost_equal(a.miny, 99.9979, places=3)
    assert_almost_equal(a.maxx, 100.0030, places=3)
    assert_almost_equal(a.maxy, 100.0030, places=3)

    e = mapnik.Box2d(100, 100, 200, 200)
    e *= 2
    eq_(e.minx, 50)
    eq_(e.miny, 50)
    eq_(e.maxx, 250)
    eq_(e.maxy, 250)

    assert_true(e.contains(50, 50))
    assert_true(e.contains(50, 250))
    assert_true(e.contains(250, 250))
    assert_true(e.contains(250, 50))

    assert_false(e.contains(49.9, 49.9))
    assert_false(e.contains(49.9, 250.1))
    assert_false(e.contains(250.1, 250.1))
    assert_false(e.contains(250.1, 49.9))

    assert_true(e.contains(e.center()))

    eq_(e.width(), 200)
    eq_(e.height(), 200)

    eq_(e.minx, 50)
    eq_(e.miny, 50)

    eq_(e.maxx, 250)
    eq_(e.maxy, 250)

    c = e.center()

    eq_(c.x, 150)
    eq_(c.y, 150)


def test_envelope_clipping():
    e1 = mapnik.Box2d(-180, -90, 180, 90)
    e2 = mapnik.Box2d(-120, 40, -110, 48)
    e1.clip(e2)
    eq_(e1, e2)

    # madagascar in merc
    e1 = mapnik.Box2d(4772116.5490, -2744395.0631, 5765186.4203, -1609458.0673)
    e2 = mapnik.Box2d(5124338.3753, -2240522.1727, 5207501.8621, -2130452.8520)
    e1.clip(e2)
    eq_(e1, e2)

    # nz in lon/lat
    e1 = mapnik.Box2d(163.8062, -47.1897, 179.3628, -33.9069)
    e2 = mapnik.Box2d(173.7378, -39.6395, 174.4849, -38.9252)
    e1.clip(e2)
    eq_(e1, e2)

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# coding=utf8
import os
import sys

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all

PYTHON3 = sys.version_info[0] == 3


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

# test discrete colorizer mode


def test_get_color_discrete():
    # setup
    colorizer = mapnik.RasterColorizer()
    colorizer.default_color = mapnik.Color(0, 0, 0, 0)
    colorizer.default_mode = mapnik.COLORIZER_DISCRETE

    colorizer.add_stop(10, mapnik.Color(100, 100, 100, 100))
    colorizer.add_stop(20, mapnik.Color(200, 200, 200, 200))

    # should be default colour
    eq_(colorizer.get_color(-50), mapnik.Color(0, 0, 0, 0))
    eq_(colorizer.get_color(0), mapnik.Color(0, 0, 0, 0))

    # now in stop 1
    eq_(colorizer.get_color(10), mapnik.Color(100, 100, 100, 100))
    eq_(colorizer.get_color(19), mapnik.Color(100, 100, 100, 100))

    # now in stop 2
    eq_(colorizer.get_color(20), mapnik.Color(200, 200, 200, 200))
    eq_(colorizer.get_color(1000), mapnik.Color(200, 200, 200, 200))

# test exact colorizer mode


def test_get_color_exact():
    # setup
    colorizer = mapnik.RasterColorizer()
    colorizer.default_color = mapnik.Color(0, 0, 0, 0)
    colorizer.default_mode = mapnik.COLORIZER_EXACT

    colorizer.add_stop(10, mapnik.Color(100, 100, 100, 100))
    colorizer.add_stop(20, mapnik.Color(200, 200, 200, 200))

    # should be default colour
    eq_(colorizer.get_color(-50), mapnik.Color(0, 0, 0, 0))
    eq_(colorizer.get_color(11), mapnik.Color(0, 0, 0, 0))
    eq_(colorizer.get_color(20.001), mapnik.Color(0, 0, 0, 0))

    # should be stop 1
    eq_(colorizer.get_color(10), mapnik.Color(100, 100, 100, 100))

    # should be stop 2
    eq_(colorizer.get_color(20), mapnik.Color(200, 200, 200, 200))

# test linear colorizer mode


def test_get_color_linear():
    # setup
    colorizer = mapnik.RasterColorizer()
    colorizer.default_color = mapnik.Color(0, 0, 0, 0)
    colorizer.default_mode = mapnik.COLORIZER_LINEAR

    colorizer.add_stop(10, mapnik.Color(100, 100, 100, 100))
    colorizer.add_stop(20, mapnik.Color(200, 200, 200, 200))

    # should be default colour
    eq_(colorizer.get_color(-50), mapnik.Color(0, 0, 0, 0))
    eq_(colorizer.get_color(9.9), mapnik.Color(0, 0, 0, 0))

    # should be stop 1
    eq_(colorizer.get_color(10), mapnik.Color(100, 100, 100, 100))

    # should be stop 2
    eq_(colorizer.get_color(20), mapnik.Color(200, 200, 200, 200))

    # half way between stops 1 and 2
    eq_(colorizer.get_color(15), mapnik.Color(150, 150, 150, 150))

    # after stop 2
    eq_(colorizer.get_color(100), mapnik.Color(200, 200, 200, 200))


def test_stop_label():
    stop = mapnik.ColorizerStop(
        1, mapnik.COLORIZER_LINEAR, mapnik.Color('red'))
    assert not stop.label
    label = u"32º C"
    if not PYTHON3:
        label = label.encode('utf8')
    stop.label = label
    assert stop.label == label, stop.label

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_loading_fontset_from_map():
    m = mapnik.Map(256, 256)
    mapnik.load_map(m, '../data/good_maps/fontset.xml', True)
    fs = m.find_fontset('book-fonts')
    eq_(len(fs.names), 2)
    eq_(list(fs.names), ['DejaVu Sans Book', 'DejaVu Sans Oblique'])

# def test_loading_fontset_from_python():
#     m = mapnik.Map(256,256)
#     fset = mapnik.FontSet('foo')
#     fset.add_face_name('Comic Sans')
#     fset.add_face_name('Papyrus')
#     eq_(fset.name,'foo')
#     fset.name = 'my-set'
#     eq_(fset.name,'my-set')
#     m.append_fontset('my-set', fset)
#     sty = mapnik.Style()
#     rule = mapnik.Rule()
#     tsym = mapnik.TextSymbolizer()
#     eq_(tsym.fontset,None)
#     tsym.fontset = fset
#     rule.symbols.append(tsym)
#     sty.rules.append(rule)
#     m.append_style('Style',sty)
#     serialized_map = mapnik.save_map_to_string(m)
#     eq_('fontset-name="my-set"' in serialized_map,True)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))
